//----------------------------------------------------------------------------
// $Id: Board.java,v 1.23 2004/07/06 16:39:31 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/go/Board.java,v $
//----------------------------------------------------------------------------

package go;

import gui.HGui;

import java.util.*;

//----------------------------------------------------------------------------

class MoveRecord
{
    public MoveRecord(Color oldToMove, Move m, Color old)
    {
        m_old = old;
        m_oldToMove = oldToMove;
        m_move = m;
    }

    public Move getMove()
    {
        return m_move;
    }

    public Color getOldColor()
    {
        return m_old;
    }

    public Color getOldToMove()
    {
        return m_oldToMove;
    }

    private Color m_old;

    private Color m_oldToMove;

    private Move m_move;
}

//----------------------------------------------------------------------------

public class GameBoard
{
    public static final int RULES_UNKNOWN = 0;

    public static final int RULES_CHINESE = 1;

    public static final int RULES_JAPANESE = 2;

    public static final int NUMBER_ROTATIONS = 8;

    public GameBoard(int boardSize)
    {
        initSize(boardSize);
    }

    public boolean bothPassed()
    {
        return isGameOver();
        /*if (m_moveNumber < 2)
            return false;
        return (getInternalMove(m_moveNumber - 1).getPoint() == null
                && getInternalMove(m_moveNumber - 2).getPoint() == null);*/
    }

    public void calcScore()
    {
        assert(isMarkCleared());
        boolean allEmpty = true;
        for (int i = 0; i < m_allPoints.length; ++i)
        {
            Point p = m_allPoints[i];
            Color c = getColor(p);
            setScore(p, Color.EMPTY);
            if (c != Color.EMPTY)
            {
                allEmpty = false;
                if (! scoreGetDead(p))
                    setScore(p, c);
            }
        }
        if (allEmpty)
            return;
        Vector territory = new Vector(getNumberPoints());
        for (int i = 0; i < m_allPoints.length; ++i)
        {
            Point p = m_allPoints[i];
            if (! getMark(p))
            {
                territory.clear();
                if (isTerritory(p, territory, Color.BLACK))
                {
                    for (int j = 0; j < territory.size(); ++j)
                        setScore((Point)territory.get(j), Color.BLACK);
                }
                else
                {
                    setMark(territory, false);
                    if (isTerritory(p, territory, Color.WHITE))
                    {
                        for (int j = 0; j < territory.size(); ++j)
                            setScore((Point)territory.get(j), Color.WHITE);
                    }
                    else
                    {
                        setMark(territory, false);
                    }
                }
            }
        }
        clearMark();
    }

    public boolean contains(Point point)
    {
        int size = getSize();
        return (point.getX() < size && point.getY() < size);
    }

    public Vector getAdjacentPoints(Point p)
    {
        Vector result = new Vector(4);
        int x = p.getX();
        int y = p.getY();
        if (x > 0)
            result.add(m_point[x - 1][y]);
        if (x < m_size - 1)
            result.add(m_point[x + 1][y]);
        if (y > 0)
            result.add(m_point[x][y - 1]);
        if (y < m_size - 1)
            result.add(m_point[x][y + 1]);
        return result;
    }

    public Color getScore(Point p)
    {
        return m_score[p.getX()][p.getY()];
    }

    public int getSize()
    {
        return m_size;
    }

    public int getCapturedB()
    {
        return m_capturedB;
    }

    public int getCapturedW()
    {
        return m_capturedW;
    }

    public Color getColor(Point p)
    {
        return m_color[p.getX()][p.getY()];
    }

    public Vector getHandicapStones(int n)
    {
        return getHandicapStones(n, m_constants);
    }

    public static Vector getHandicapStones(int size, int n)
    {
        return getHandicapStones(n, new Constants(size));
    }

    public Move getMove(int i)
    {
        return m_moves.get(i + m_setupNumber).getMove();
    }

    public Point getPoint(int i)
    {
        return m_allPoints[i];
    }

    public Point getPoint(int x, int y)
    {
        return m_point[x][y];
    }

    public Move getInternalMove(int i)
    {
        return m_moves.get(i).getMove();
    }

    public int getInternalNumberMoves()
    {
        return m_setupNumber;
    }

    public int getMoveNumber()
    {
        return m_moveNumber - m_setupNumber;
    }

    public int getNumberPoints()
    {
        return m_allPoints.length;
    }

    public Vector getSetupStonesBlack()
    {
        return m_setupStonesBlack;
    }

    public Vector getSetupStonesWhite()
    {
        return m_setupStonesWhite;
    }

    public void getStones(Point p, Color color, Vector stones)
    {
        assert(isMarkCleared());
        findStones(p, color, stones);
        setMark(stones, false);
        assert(isMarkCleared());
    }

    public Color getToMove()
    {
        if (m_game_over) return Color.EMPTY;
        return m_toMove;
    }

    public void initSize(int size)
    {
        m_size = size;
        m_color = new Color[m_size][m_size];
        m_mark = new boolean[m_size][m_size];
        m_dead = new boolean[m_size][m_size];
        m_score = new Color[m_size][m_size];
        m_point = new Point[m_size][m_size];
        frec = new int[m_size][m_size]; // for determining a win
        m_capturedB = 0;
        m_capturedW = 0;
        initAllPoints();
        m_constants = new Constants(size);
        newGame();
    }

    public boolean isEdgeLine(int i)
    {
        return (i == 0 || i == m_size - 1);
    }

    public boolean isHandicapLine(int i)
    {
        return (i == m_constants.m_handicapLine1
                || i == m_constants.m_handicapLine2
                || i == m_constants.m_handicapLine3);
    }

    public boolean isHandicap(Point p)
    {
        int x = p.getX();
        int y = p.getY();
        return (isHandicapLine(x) && isHandicapLine(y));
    }

    public boolean isModified()
    {
        return (m_moves.size() > 0 || m_setupNumber > 0);
    }

    public boolean isSuicide(Point point, Color toMove)
    {
        return false;
    }

    public void newGame()
    {
        for (int i = 0; i < m_allPoints.length; ++i)
            setColor(m_allPoints[i], Color.EMPTY);
        m_moves.clear();
        m_moveNumber = 0;
        m_setupNumber = 0;
        m_capturedB = 0;
        m_capturedW = 0;
        m_toMove = Color.BLACK;
        m_setupStonesBlack.clear();
        m_setupStonesWhite.clear();
    }

    /**
     *  parameter condition: x1<=x2; (pt1 = links unten; pt2=rechts oben)
     */
    private void swapDiag(int x1, int y1, int x2, int y2) {
        int dx=1;
        int dy=-1;
        int diff1=Math.abs(x1-x2);
        int diff2=Math.abs(y1-y2);
        if (diff1!=diff2) return;
        for (int i=diff1; i>=0; i-=2) {
            if (!(x1==x2 && y1==y2)) {
                Color t=m_color[x1][y1];
                m_color[x1][y1]=m_color[x2][y2].otherColor();
                m_color[x2][y2]=t.otherColor();
            }
            else { // same position (=middle)
                m_color[x1][y1]=m_color[x1][y1].otherColor();
            }
            x1+=dx; x2-=dx;
            y1+=dy; y2-=dy;
        }
    }

    private void swapBoard() {
        for (int i=1; i<m_size; i++) {
            swapDiag(0,i,i,0);
        }
        for (int i=1; i<m_size-1; i++) {
            swapDiag(i,m_size-1,m_size-1,i);
        }
    }

    boolean m_game_over=false;

    public boolean isGameOver() {
        return m_game_over;
    }

    /*public Color colToMove() {
      return m_toMove;
    }*/

    public boolean play(Move m) {
        if (m_game_over) {
          System.err.println("FAIL in play(): m_game_over");
          return false;
        }
        Point p = m.getPoint();
        Color col = m.getColor();
        //if (color!=m_toMove) return false;
        if (getToMove()!=col) {
          System.err.println("FAIL in play(): Wrong color to move!");
          return false;
        }
        Color old_field=Color.EMPTY;
        if (m.getSpecialMove()!=Move.MOVE_NORMAL) {
            int spc = m.getSpecialMove();
            // pass einfach ignorieren (hier) ...
            if (spc == Move.MOVE_SWAP) {
                swapBoard();
            }
            if (spc == Move.MOVE_RESIGN) {
                m_game_over=true;
            }
        }
        else { // REGULAR Move:
            //#fixme: check if onBoard!
            old_field = getColor(p);
            if (old_field!=Color.EMPTY) { //cannot put on top of prev move
              System.err.println("FAIL in play(): cell already occupied!");
              return false;
            }
            setColor(p, col);
            m_game_over=(getConnectWinner()!=Color.EMPTY);
        }
        m_moves.setSize(m_moveNumber);
        m_moves.add(new MoveRecord(m_toMove, m, old_field));
        ++m_moveNumber;
        m_toMove = col.otherColor();
        return true;
    }

    public boolean undo() {
        m_game_over=false;
        if (getMoveNumber() == 0)
            return false;
        --m_moveNumber;
        MoveRecord r = m_moves.get(m_moveNumber);
        m_moves.setSize(m_moveNumber);
        Move m = r.getMove();
        Color c = m.getColor();
        Color otherColor = c.otherColor();
        Point p = m.getPoint();
        if (p != null)
        {
            setColor(p, r.getOldColor());
        }
        else { // swap
            int spc = m.getSpecialMove();
            // pass einfach ignorieren (hier) ...
            if (spc == Move.MOVE_SWAP) {
                swapBoard();
            }
            if (spc == Move.MOVE_RESIGN) {
                // do nothing
            }
        }
        m_toMove = r.getOldToMove();
        return true;
    }

    public boolean isOnBoard(int x, int y)
    {
        return Point.isOnBoard(m_size, x, y);
    }

    public Point rotate(int rotationIndex, Point point)
    {
        assert(rotationIndex < NUMBER_ROTATIONS);
        if (point == null)
            return null;
        int x = point.getX();
        int y = point.getY();
        int size = getSize();
        switch (rotationIndex)
        {
        case 0:
            return getPoint(x, y);
        case 1:
            return getPoint(size - x - 1, y);
        case 2:
            return getPoint(x, size - y - 1);
        case 3:
            return getPoint(y, x);
        case 4:
            return getPoint(size - y - 1, x);
        case 5:
            return getPoint(y, size - x - 1);
        case 6:
            return getPoint(size - x - 1, size - y - 1);
        case 7:
            return getPoint(size - y - 1, size - x - 1);
        default:
            return getPoint(x, y);
        }
    }

    public void scoreBegin(Point[] isDeadStone)
    {
        for (int i = 0; i < m_allPoints.length; ++i)
            scoreSetDead(m_allPoints[i], false);
        if (isDeadStone != null)
            for (int i = 0; i < isDeadStone.length; ++i)
                scoreSetDead(isDeadStone[i], true);
        calcScore();
    }

    /** Mark point as dead for scoring. */
    public void scoreSetDead(Point p, boolean value)
    {
        m_dead[p.getX()][p.getY()] = value;
    }

    public Score scoreGet(float komi, int rules)
    {
        Score s = new Score();
        s.m_rules = rules;
        s.m_komi = komi;
        s.m_capturedBlack = m_capturedB;
        s.m_capturedWhite = m_capturedW;
        int areaDiff = 0;
        int territoryDiff = 0;
        for (int i = 0; i < m_allPoints.length; ++i)
        {
            Point p = m_allPoints[i];
            Color c = getColor(p);
            Color sc = getScore(p);
            if (sc == Color.BLACK)
            {
                ++s.m_areaBlack;
                ++areaDiff;
            }
            else if (sc == Color.WHITE)
            {
                ++s.m_areaWhite;
                --areaDiff;
            }
            if (c == Color.EMPTY)
            {
                if (sc == Color.BLACK)
                {
                    ++s.m_territoryBlack;
                    ++territoryDiff;
                }
                else if (sc == Color.WHITE)
                {
                    ++s.m_territoryWhite;
                    --territoryDiff;
                }
            }
            if (c == Color.BLACK && sc == Color.WHITE)
            {
                ++s.m_capturedBlack;
                ++s.m_territoryWhite;
                --territoryDiff;
            }
            if (c == Color.WHITE && sc == Color.BLACK)
            {
                ++s.m_capturedWhite;
                ++s.m_territoryBlack;
                ++territoryDiff;
            }
        }
        s.m_resultChinese = areaDiff - komi;
        s.m_resultJapanese =
            s.m_capturedWhite - s.m_capturedBlack + territoryDiff - komi;
        if (rules == RULES_JAPANESE)
            s.m_result = s.m_resultJapanese;
        else
            s.m_result = s.m_resultChinese;
        return s;
    }

    public boolean scoreGetDead(Point p)
    {
        return m_dead[p.getX()][p.getY()];
    }

    public void setToMove(Color toMove)
    {
        m_toMove = toMove;
    }

    // if it is not a special move
    public Point getLastMove() {
        int sz=m_moves.size();
        if (sz==0) return null;
        MoveRecord last=m_moves.get(sz-1);
        Move m=last.getMove();
        if (m==null) return null;
        Point p=m.getPoint();
        if (p==null) return null;
        return p;
    }

    /** Set a stone on the board.
        Will remove dead stones.
        Requires: getMoveNumber() == 0
    */
    public void setup(Move m)
    {
        assert(getMoveNumber() == 0);
        play(m);
        if (m.getColor() == Color.BLACK)
            m_setupStonesBlack.add(m.getPoint());
        else if (m.getColor() == Color.WHITE)
            m_setupStonesWhite.add(m.getPoint());
        else
            assert(false);
        ++m_setupNumber;
    }


    //final int neighb[][] = { {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,-1} }; // neighbours
      final int neighb[][] = { {1,0}, {0,1}, {-1,1}, {-1,0}, {0,-1}, {1,-1} }; // neighbours

    //****************************************************************************
    //******** CHECK bridge (get Hex Winner)

    // USES GLOBAL VARIABLE: frec
    // frec: 1=already visited from a corner, 0=never visited before
    // entries in frec from earlier passes do not matter!!

    private int frec[][];   // INTERNAL: field for determining a win

    boolean floodfill_bridge(int x, int y, Color c)
    {
        if (isOnBoard(x,y) && (frec[x][y]==0) && (m_color[x][y]==c)) // new field found
        {
            if ((c==Color.WHITE && x==m_size-1) ||
                (c==Color.BLACK && y==m_size-1)) return true;

            frec[x][y]=1; // visited

            // check neighbours also
            for(int k=0;k<=5;k++) {
                int cx=x+neighb[k][0];
                int cy=y+neighb[k][1];

                if (floodfill_bridge(cx, cy, c)) return true;
            }
        }
        return false;
    }

    public boolean bridge_exists(Color c)
    {
        // clear frec
        for(int i=0;i<m_size;i++)
            for(int j=0;j<m_size;j++)
                frec[i][j]=0;

        if (c==Color.WHITE) { // horiz
          for(int i=0; i<m_size; i++) { // along left edge
            if (floodfill_bridge(0,i,c)) return true;
          }
        }
        else if (c==Color.BLACK) { // vert
          for(int i=0; i<m_size; i++) { // along top edge
            if (floodfill_bridge(i,0,c)) return true;
          }
        }

        return false;
    }

    public Color getConnectWinner()
    {
             if (bridge_exists(Color.WHITE)) return Color.WHITE;
        else if (bridge_exists(Color.BLACK)) return Color.BLACK;
        else return Color.EMPTY;
    }

    /** Some values that are constant fpr a given board size. */
    private static class Constants
    {
        public Constants(int size)
        {
            m_handicapLine1 = -1;
            m_handicapLine2 = -1;
            m_handicapLine3 = -1;
            if (size >= 13)
            {
                m_handicapLine1 = 3;
                m_handicapLine3 = size - 4;
            }
            else if (size >= 8)
            {
                m_handicapLine1 = 2;
                m_handicapLine3 = size - 3;
            }
            if (size >= 11 && size % 2 != 0)
                m_handicapLine2 = size / 2;
        }

        public int m_handicapLine1;

        public int m_handicapLine2;

        public int m_handicapLine3;
    }

    private boolean m_mark[][];

    private boolean m_dead[][];

    private int m_size;

    private int m_capturedB;

    private int m_capturedW;

    private int m_moveNumber;

    private int m_setupNumber;

    private Vector<MoveRecord> m_moves = new Vector<MoveRecord>(361, 361); //int initialCapacity, int capacityIncrement

    private Vector m_setupStonesBlack = new Vector();

    private Vector m_setupStonesWhite = new Vector();

    private Color m_color[][];

    private Color m_score[][];

    private Color m_toMove;

    private Constants m_constants;

    private Point m_allPoints[];

    private Point m_point[][];

    private void checkKill(Point p, Color color, Vector killed)
    {
        assert(isMarkCleared());
        Vector stones = new Vector();
        if (isDead(p, color, stones))
        {
            killed.addAll(stones);
            for (int i = 0; i < stones.size(); ++i)
                setColor((Point)stones.get(i), Color.EMPTY);
        }
        setMark(stones, false);
        assert(isMarkCleared());
    }

    private void clearMark()
    {
        for (int i = 0; i < m_allPoints.length; ++i)
        {
            Point p = m_allPoints[i];
            setMark(p, false);
        }
    }

    private void findStones(Point p, Color color, Vector stones)
    {
        Color c = getColor(p);
        if (c != color)
            return;
        if (getMark(p))
            return;
        setMark(p, true);
        stones.add(p);
        Vector adj = getAdjacentPoints(p);
        for (int i = 0; i < adj.size(); ++i)
            findStones((Point)(adj.get(i)), color, stones);
    }

    private static Vector getHandicapStones(int n, Constants constants)
    {
        Vector result = new Vector(9);
        if (n == 0)
            return result;
        if (constants.m_handicapLine1 < 0)
            return null;
        if (n > 4 && constants.m_handicapLine2 < 0)
            return null;
        if (n >= 1)
            result.add(new Point(constants.m_handicapLine1,
                                 constants.m_handicapLine1));
        if (n >= 2)
            result.add(new Point(constants.m_handicapLine3,
                                 constants.m_handicapLine3));
        if (n >= 3)
            result.add(new Point(constants.m_handicapLine1,
                                 constants.m_handicapLine3));
        if (n >= 4)
            result.add(new Point(constants.m_handicapLine3,
                                 constants.m_handicapLine1));
        if (n >= 5)
            if (n % 2 != 0)
            {
                result.add(new Point(constants.m_handicapLine2,
                                     constants.m_handicapLine2));
                --n;
            }
        if (n >= 5)
            result.add(new Point(constants.m_handicapLine1,
                                 constants.m_handicapLine2));
        if (n >= 6)
            result.add(new Point(constants.m_handicapLine3,
                                 constants.m_handicapLine2));
        if (n >= 7)
            result.add(new Point(constants.m_handicapLine2,
                                 constants.m_handicapLine1));
        if (n >= 8)
            result.add(new Point(constants.m_handicapLine2,
                                 constants.m_handicapLine3));
        return result;
    }

    private boolean getMark(Point p)
    {
        return m_mark[p.getX()][p.getY()];
    }

    private void initAllPoints()
    {
        m_allPoints = new Point[m_size * m_size];
        int i = 0;
        for (int x = 0; x < m_size; ++x)
            for (int y = 0; y < m_size; ++y)
            {
                Point p = new Point(x, y);
                m_allPoints[i++] = p;
                m_point[x][y] = p;
            }
    }

    private boolean isDead(Point p, Color color, Vector stones)
    {
        Color c = getColor(p);
        if (c == Color.EMPTY)
            return false;
        if (c != color)
            return true;
        if (getMark(p))
            return true;
        setMark(p, true);
        stones.add(p);
        Vector adj = getAdjacentPoints(p);
        for (int i = 0; i < adj.size(); ++i)
            if (! isDead((Point)(adj.get(i)), color, stones))
                return false;
        return true;
    }

    private boolean isMarkCleared()
    {
        for (int i = 0; i < m_allPoints.length; ++i)
            if (getMark(m_allPoints[i]))
                 return false;
        return true;
    }

    private boolean isTerritory(Point p, Vector territory, Color color)
    {
        Color c = getColor(p);
        if (c == color.otherColor() && ! scoreGetDead(p))
            return false;
        if (c == color)
        {
            if (scoreGetDead(p))
                return false;
            else
                return true;
        }
        if (getMark(p))
            return true;
        setMark(p, true);
        territory.add(p);
        Vector adj = getAdjacentPoints(p);
        for (int i = 0; i < adj.size(); ++i)
            if (! isTerritory((Point)(adj.get(i)), territory, color))
                return false;
        return true;
    }

    private void setColor(Point p, Color color)
    {
        m_color[p.getX()][p.getY()] = color;
    }

    private void setMark(Point p, boolean value)
    {
        m_mark[p.getX()][p.getY()] = value;
    }

    private void setMark(Vector points, boolean value)
    {
        int size = points.size();
        for (int i = 0; i < size; ++i)
            setMark((Point)points.get(i), value);
    }

    private void setScore(Point p, Color c)
    {
        assert(c != null);
        m_score[p.getX()][p.getY()] = c;
    }
}

//----------------------------------------------------------------------------
