//----------------------------------------------------------------------------
// $Id: Move.java,v 1.6 2004/05/10 22:49:05 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/go/Move.java,v $
//----------------------------------------------------------------------------

package go;

import java.util.*;

//----------------------------------------------------------------------------

public class Move
{
    public static final int MOVE_NORMAL=0;
    public static final int MOVE_PASS=1;
    public static final int MOVE_SWAP=2;
    public static final int MOVE_SWAP_SIDES=5;
    public static final int MOVE_RESIGN=3;
    public static final int MOVE_FORFEIT=4;

    public Move(int specialMove, Color c)
    {
        m_color = c;
        m_point = null;
        m_special = specialMove;
    }

    public Move(Point p, Color c)
    {
        m_color = c;
        m_point = p;
        if (p!=null) m_special = 0;
                else m_special = MOVE_PASS;
    }

    /** Fill a list of moves with pass moves.
        The resulting list will contain all moves of the original list
        in the same order, but ensure it starts with a move of color toMove
        and have no subsequent moves of the same color.
    */
    public static Vector fillPasses(Vector moves, Color toMove)
    {
        Vector result = new Vector(moves.size() * 2);
        if (moves.size() == 0)
            return result;
        for (int i = 0; i < moves.size(); ++i)
        {
            Move move = (Move)moves.get(i);
            if (move.getColor() != toMove)
                result.add(new Move(null, toMove));
            result.add(move);
            toMove = move.getColor().otherColor();
        }
        return result;
    }

    public Color getColor()
    {
        return m_color;
    }

    public Point getPoint()
    {
        return m_point;
    }

    public int getSpecialMove()
    {
        return m_special;
    }

    public void setSpecialMove(int new_special)
    {
        m_special=new_special;
    }

    public void setColor(Color new_col)
    {
        m_color=new_col;
    }

    /*
     * does nothing, if special move
     **/
    public void swapCoords() {
      if (m_special==MOVE_NORMAL) {
        int x=m_point.getX();
        int y=m_point.getY();
        m_point=new Point(y,x);
      }
    }

    public int hashCode()
    {
        int code = 0;
        if (m_point != null)
            code = (m_point.getX() << 17) | (m_point.getY() << 1);
        if (m_color == Color.BLACK)
            code |= 1;
        return code;
    }

    //++
    public boolean equals(Object other)
    {
        if (this == other) return true;
        if (other==null) return false; //unnecessary?

        if (other instanceof Move) {
            Move o2 = (Move)other;
            if (m_color != o2.m_color) return false;
            if (m_special != o2.m_special) return false;
            if (m_special != MOVE_NORMAL) return true; //both are same special move!
            if (m_point == null) return (o2.m_point == null);
            if (o2.m_point == null) return false; //unnecessary?
            return (m_point.equals(o2.m_point));
        }
        return false;
    }

    // compat (for implicit calls)
    public String toString() {
      return toString(false);
    }

    // sgf has explicit swap-... written to it
    public String toString(boolean forSgf) {
        if (m_point == null) {
            if (m_special==MOVE_PASS) return "pass";
            if (forSgf) {
              //if (m_special==MOVE_SWAP) return "swap-pieces";
              if (m_special==MOVE_SWAP) return "swap"; //for now, write this
              if (m_special==MOVE_SWAP_SIDES) return "swap-sides";
            } else {
              if (m_special==MOVE_SWAP) return "swap";
            }
            if (m_special==MOVE_SWAP_SIDES) return "swap-sides"; //should never happen (because it is converted on sgf-loading)
            if (m_special==MOVE_RESIGN) return "resign";
            if (m_special==MOVE_FORFEIT) return "forfeit";
            return "";
        }
        else
            return (m_point.toString());
    }

    // for gtp + sgf !!
    public static Move parseMove(Color col, String s, int boardSize, boolean sgfLoading) throws Exception
    {
        s = s.trim().toLowerCase();
        s = s.replaceAll("|draw",""); // for little golem
        if (s.equals("")) return new Move(MOVE_PASS, col);
        if (s.equals("pass")) return new Move(MOVE_PASS, col);
        if (s.equals("swap")) return new Move(MOVE_SWAP, col);
        if (s.equals("swap-pieces")) return new Move(MOVE_SWAP, col); //also accepted (offical standard)
        if (sgfLoading && s.equals("swap-sides")) return new Move(MOVE_SWAP_SIDES, col);
        if (s.equals("resign")) return new Move(MOVE_RESIGN, col);
        if (s.equals("forfeit")) return new Move(MOVE_FORFEIT, col);
        return new Move(Point.parsePoint(s, boardSize), col);
    }

    private Color m_color;

    private Point m_point;

    private int m_special;  // if m_point==null

}

//----------------------------------------------------------------------------
