//----------------------------------------------------------------------------
// $Id: Point.java,v 1.7 2004/05/10 22:49:05 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/go/Point.java,v $
//----------------------------------------------------------------------------

package go;

//----------------------------------------------------------------------------

public class Point
{
    public Point(int x, int y)
    {
        set(x, y);
    }

    public void set(int x, int y)
    {
        assert(x >= 0);
        assert(y >= 0);
        m_x = x;
        m_y = y;
    }

    public int getX()
    {
        return m_x;
    }

    public int getY()
    {
        return m_y;
    }

    public static boolean equals(Point point1, Point point2)
    {
        if (point1 == null && point2 == null)
            return true;
        if (point1 == null || point2 == null)
            return false;
        return point1.equals(point2);
    }

    public boolean equals(Object other)
    {
        if (this == other) return true;
        if (other==null) return false; //unnecessary?

        if (other instanceof Point) {
            Point o2 = (Point)other;
            return (m_x == o2.m_x && m_y == o2.m_y);
        }
        return false;
    }

    public int hashCode()
    {
        return (m_x << 16) | m_y;
    }

    public void left()
    {
        if (m_x > 0)
            --m_x;
    }

    public void down()
    {
        if (m_y > 0)
            --m_y;
    }

    public void right(int max)
    {
        if (m_x < max - 1)
            ++m_x;
    }

    public void up(int max)
    {
        if (m_y < max - 1)
            ++m_y;
    }


    // lowercase; used for everything
    // * is used for sending in gtp protocol !!
    public String toString()
    {
        return ((char)('a'+m_x)) + Integer.toString(m_y + 1);
    }

    // lowercase; used for everything
    public static String toString(Point point)
    {
        if (point == null) return "special";
        return point.toString();
    }

    // littlegolem format can have two chars, like "ag"
    public static Point parsePoint(String s, int boardSize) throws Exception
    {
        s = s.trim().toLowerCase();
        if (s.length() < 2) throw new Exception("Invalid point or move: "+s);
        char xChar = s.charAt(0);
        char yChar = s.charAt(1);
        if (xChar<'a' || xChar>'z') throw new Exception("Invalid point or move: "+s);
        int x = xChar - 'a';
        try {
          int y;
          if (yChar>='a' && yChar<='z') {
            y = yChar - 'a';
          } else {
            y = Integer.parseInt(s.substring(1)) - 1;
          }
          if (!isOnBoard(boardSize, x, y))
            throw new Exception("Invalid point or move(!isOnBoard): "+s);
          return new Point(x, y);
        }
        catch (NumberFormatException e) {
          throw new Exception("Invalid point or move: "+s);
        }
    }

    private int m_x;

    private int m_y;

    // board_size default = 10
    public static boolean isOnBoard(int board_size, int x, int y)
    {
        return (x>=0 && y>=0 && x<board_size && y<board_size);
    }

}

//----------------------------------------------------------------------------
