//----------------------------------------------------------------------------
// $Id: GameInfo.java,v 1.20 2004/05/10 22:49:06 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/gui/GameInfo.java,v $
//----------------------------------------------------------------------------

package gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import game.*;
import go.*;
import go.Color;
import utils.*;

//----------------------------------------------------------------------------

class GameInfo
    extends JPanel
    implements ActionListener
{
    public GameInfo(TimeControl timeControl)
    {
        super(new GridLayout(0, 2, GuiUtils.SMALL_PAD, 0));
        m_timeControl = timeControl;
        m_bsize = addEntry("Boardsize");
        m_number = addEntry("Moves");
        m_move = addEntry("To Play");
        m_last = addEntry("Last Move");
        //m_captB = addEntry("Captured Black");
        //m_captW = addEntry("Captured White");
        m_timeB = addEntry("Time Black");
        m_timeW = addEntry("Time White");
        m_timeB.setText("00:00");
        m_timeW.setText("00:00");
        new javax.swing.Timer(1000, this).start();
    }

    public void actionPerformed(ActionEvent evt)
    {
        if (m_timeControl.isRunning())
        {
            setTime(go.Color.BLACK);
            setTime(go.Color.WHITE);
        }
    }

    public boolean update(Node node, go.GameBoard board)
    {
        boolean ret=false;

        if (board.getToMove() == go.Color.BLACK)
            //m_move.setText("Black");
            m_move.setText("Red"); //PHIL
        else if (board.getToMove() == go.Color.WHITE)
            //m_move.setText("White");
            m_move.setText("Blue"); //PHIL
        else
            m_move.setText("");
        //String capturedB = Integer.toString(board.getCapturedB());
        //m_captB.setText(capturedB);
        //String capturedW = Integer.toString(board.getCapturedW());
        //m_captW.setText(capturedW);
        int moveNumber = node.getMoveNumber();
        String numberString = Integer.toString(moveNumber);
        int movesLeft = node.getMovesLeft();
        if (movesLeft > 0)
            numberString += "/" + (moveNumber + movesLeft);
        m_number.setText(numberString);
        String lastMove = "";
        Move move = node.getMove();
        if (move != null)
        {
            go.Color c = move.getColor();
            go.Point p = move.getPoint();
            //            lastMove += move.toString().toUpperCase();
            lastMove = (c == go.Color.BLACK ? "Red" : "Blue") + " "; //PHIL
            if (board.isGameOver() && board.getConnectWinner()!=Color.EMPTY) {   // PHIL
              lastMove+="[CONNECT]";
              ret=true;
            } else {
                ret=false;
            }
        }
        m_bsize.setText(""+board.getSize());
        m_last.setText(lastMove);
        float timeLeftBlack = node.getTimeLeftBlack();
        int movesLeftBlack = node.getMovesLeftBlack();
        if (! Float.isNaN(timeLeftBlack))
            m_timeB.setText(TimeControl.getTimeString(timeLeftBlack,
                                                      movesLeftBlack));
        float timeLeftWhite = node.getTimeLeftWhite();
        int movesLeftWhite = node.getMovesLeftWhite();
        if (! Float.isNaN(timeLeftWhite))
            m_timeW.setText(TimeControl.getTimeString(timeLeftWhite,
                                                      movesLeftWhite));

        return(ret);
    }

    private JLabel m_move;

    private JLabel m_number;

    private JLabel m_last;

    private JLabel m_bsize;

    private JLabel m_timeB;

    private JLabel m_timeW;

    private TimeControl m_timeControl;

    private JLabel addEntry(String text)
    {
        JLabel label = new JLabel(text);
        label.setHorizontalAlignment(SwingConstants.LEFT);
        add(label);
        JLabel entry = new JLabel(" ");
        entry.setHorizontalAlignment(SwingConstants.LEFT);
        entry.setBorder(BorderFactory.createLoweredBevelBorder());
        add(entry);
        return entry;
    }

    private void setTime(go.Color c)
    {
        String text = m_timeControl.getTimeString(c);
        if (text == null)
            text = " ";
        if (c == go.Color.BLACK)
            m_timeB.setText(text);
        else
            m_timeW.setText(text);
    }
}

//----------------------------------------------------------------------------
