//----------------------------------------------------------------------------
// $Id: Board.java,v 1.71 2004/05/10 22:49:06 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/gui/Board.java,v $
//----------------------------------------------------------------------------

package gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.util.*;

import javax.swing.*;
import go.*;
import utils.GuiUtils;
import java.io.File;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.Color;

//----------------------------------------------------------------------------

class BoardLabel
    extends JLabel
{
    public BoardLabel(String text)
    {
        super(text, JLabel.CENTER);
        setForeground(java.awt.Color.DARK_GRAY);
    }

    public void paintComponent(Graphics graphics)
    {
        if (getSize().height < getFont().getSize())
            return;
        super.paintComponent(graphics);
    }
}

public class GuiBoard
    extends JPanel
    implements FocusListener, Printable,
        MouseListener  // neu
{
    public interface Listener
    {
        void fieldClicked(go.Point p, boolean modifiedSelect);
    }

    public Color ColorBlack = Color.blue; // PHIL
    public Color ColorWhite = Color.red; // PHIL
    public int DrawLoose = 0; //PHIL


    private Image img1;

    public GuiBoard(go.GameBoard board)
    {
        super(new GridBagLayout());

        // PHIL
        ClassLoader classLoader = ClassLoader.getSystemClassLoader();
        java.net.URL url = classLoader.getResource("images/" + "smiley.png");

        img1 = Toolkit.getDefaultToolkit().getImage(url);

        ColorWhite = Color.blue;  // PHIL
        ColorBlack = Color.red;
        DrawLoose = 0;
        m_board = board;
        setPreferredFieldSize();
        initSize(m_board.getSize());
        setFocusable(true);
        addFocusListener(this);
        addMouseListener(this); // NEU!!
    }

    public void clearAll()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            clearInfluence(p);
            setFieldBackground(p, null);
        }
        clearAllCrossHair();
        clearAllMarkup();
        clearAllSelect();
        clearAllStrings();
        clearAllTerritory();
        clearLastMove();
        if (m_variationShown)
        {
            updateFromGoBoard();
            m_variationShown = false;
        }
    }

    public void clearAllCrossHair()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            setCrossHair(m_board.getPoint(i), false);
    }

    public void clearAllMarkup()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            setMarkup(m_board.getPoint(i), false);
    }

    public void clearAllSelect()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            setSelect(m_board.getPoint(i), false);
    }

    public void clearAllStrings()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            setString(m_board.getPoint(i), "");
    }

    public void clearAllTerritory()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            setTerritory(m_board.getPoint(i), go.Color.EMPTY);
    }

    public void clearInfluence(go.Point p)
    {
        //getField(p).clearInfluence();
    }

    public void focusGained(FocusEvent event)
    {
        //setFocusPoint(m_focusPoint);
    }

    public void focusLost(FocusEvent event)
    {
    }

    public void fieldClicked(go.Point p, boolean modifiedSelect)
    {
        if (m_listener != null)
            m_listener.fieldClicked(p, modifiedSelect);
    }

    public go.GameBoard getBoard()
    {
        return m_board;
    }

    public boolean[][] getMarkups()
    {
        return null; // result
    }

    public Dimension getMinimumFieldSize()
    {
        return m_minimumFieldSize;
    }

    public Dimension getPreferredFieldSize()
    {
        return m_preferredFieldSize;
    }

    public boolean[][] getSelects()
    {
        return null; // result
    }

    /*
    public boolean getShowCursor()
    {
        return m_showCursor;
    }
    */

    public String[][] getStrings()
    {
        return null; // result
    }

    public void initSize(int size)
    {
        m_board.initSize(size);
        m_hexBoard = new HexBoard(size);
        m_lastMove = null;
        repaint();
    }

    public void markLastMove(go.Point point)
    {
        //clearLastMove();
        m_lastMove = point;
        repaint();
    }


    public void paintImmediately(go.Point point)
    {
        repaint(getVisibleRect());
    }

    public int print(Graphics g, PageFormat format, int page)
        throws PrinterException
    {
        if (page >= 1)
        {
            return Printable.NO_SUCH_PAGE;
        }
        double width = getSize().width;
        double height = getSize().height;
        double pageWidth = format.getImageableWidth();
        double pageHeight = format.getImageableHeight();
        double scale = 1;
        if (width >= pageWidth)
            scale = pageWidth / width;
        double xSpace = (pageWidth - width * scale) / 2;
        double ySpace = (pageHeight - height * scale) / 2;
        Graphics2D g2d = (Graphics2D)g;
        g2d.translate(format.getImageableX() + xSpace,
                      format.getImageableY() + ySpace);
        g2d.scale(scale, scale);
        print(g2d);
        return Printable.PAGE_EXISTS;
    }

    public void resetBoard()
    {
        if (! m_needsReset)
            return;
        clearAll();
        m_needsReset = false;
    }

    public void scoreBegin(go.Point[] isDeadStone)
    {
        m_board.scoreBegin(isDeadStone);
        if (isDeadStone != null)
            for (int i = 0; i < isDeadStone.length; ++i)
                setCrossHair(isDeadStone[i], true);
        calcScore();
    }

    public void scoreSetDead(go.Point p)
    {
        go.Color c = m_board.getColor(p);
        if (c == go.Color.EMPTY)
            return;
        Vector stones = new Vector(m_board.getNumberPoints());
        m_board.getStones(p, c, stones);
        boolean dead = ! m_board.scoreGetDead((go.Point)(stones.get(0)));
        for (int i = 0; i < stones.size(); ++i)
        {
            go.Point stone = (go.Point)stones.get(i);
            m_board.scoreSetDead(stone, dead);
            setCrossHair(stone, dead);
        }
        calcScore();
    }

    public void setFocus()
    {
        int moveNumber = m_board.getMoveNumber();
        if (moveNumber > 0)
        {
            Move m = m_board.getMove(moveNumber - 1);
            go.Point lastMove = m.getPoint(); repaint();
            if (lastMove != null && m.getColor() != go.Color.EMPTY)
                setFocusPoint(lastMove);
        }
        else if (m_board.getInternalNumberMoves() == 0)
        {
            int size = m_board.getSize();
            if (size > 0)
                setFocusPoint(m_board.getPoint(size / 2, size / 2));
        }
    }

    public void setFocusPoint(go.Point point)
    {
    }

    public void setFieldBackground(go.Point p, java.awt.Color color)
    {
        //getField(p).setFieldBackground(color);
        m_needsReset = true;
    }

    public void setCrossHair(go.Point p, boolean crossHair)
    {
        //getField(p).setCrossHair(crossHair);
        m_needsReset = true;
    }

    public void setInfluence(go.Point p, double value)
    {
        //getField(p).setInfluence(value);
        m_needsReset = true;
    }

    public void setListener(Listener l)
    {
        m_listener = l;
    }

    public void setMarkup(go.Point p, boolean markup)
    {
        //getField(p).setMarkup(markup);
        m_needsReset = true;
    }

    /*
    public void setShowCursor(boolean showCursor)
    {
        m_showCursor = showCursor;
    }
    */

    public void setSelect(go.Point p, boolean select)
    {
        //getField(p).setSelect(select);
        m_needsReset = true;
    }

    public void setString(go.Point p, String s)
    {
        //getField(p).setString(s);
        m_needsReset = true;
    }

    public void setTerritory(go.Point p, go.Color color)
    {
        //getField(p).setTerritory(color);
        m_needsReset = true;
    }

    public void showBWBoard(String[][] board)
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            String s = board[p.getX()][p.getY()].toLowerCase();
            if (s.equals("b") || s.equals("black"))
                setTerritory(p, go.Color.BLACK);
            else if (s.equals("w") || s.equals("white"))
                setTerritory(p, go.Color.WHITE);
            else
                setTerritory(p, go.Color.EMPTY);
        }
    }

    public void showColorBoard(String[][] board)
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            String s = board[p.getX()][p.getY()];
            java.awt.Color c = null;
            if (s.equals("blue"))
                c = java.awt.Color.blue;
            else if (s.equals("cyan"))
                c = java.awt.Color.cyan;
            else if (s.equals("green"))
                c = java.awt.Color.green;
            else if (s.equals("gray"))
                c = java.awt.Color.lightGray;
            else if (s.equals("magenta"))
                c = java.awt.Color.magenta;
            else if (s.equals("pink"))
                c = java.awt.Color.pink;
            else if (s.equals("red"))
                c = java.awt.Color.red;
            else if (s.equals("yellow"))
                c = java.awt.Color.yellow;
            else if (s.equals("black"))
                c = java.awt.Color.black;
            else if (s.equals("white"))
                c = java.awt.Color.white;
            setFieldBackground(p, c);
        }
    }

    public void showDoubleBoard(double[][] board, double scale)
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            double d = board[p.getX()][p.getY()] * scale;
            setInfluence(p, d);
        }
    }

    public void showPointList(go.Point pointList[])
    {
        clearAllMarkup();
        for (int i = 0; i < pointList.length; ++i)
        {
            go.Point p = pointList[i];
            if (p != null)
                setMarkup(p, true);
        }
    }

    public void showPointStringList(Vector pointList, Vector stringList)
    {
        clearAllStrings();
        for (int i = 0; i < pointList.size(); ++i)
        {
            go.Point point = (go.Point)pointList.get(i);
            String string = (String)stringList.get(i);
            if (point != null)
                setString(point, string);
        }
    }

    public void showStringBoard(String[][] board)
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            setString(p, board[p.getX()][p.getY()]);
        }
    }

    public void showVariation(go.Point[] variation, go.Color toMove)
    {
        clearAllStrings();
        updateFromGoBoard();
        for (int i = 0; i < variation.length; ++i)
        {
            go.Point point = variation[i];
            if (point != null)
            {
                setColor(point, toMove);
                setString(point, Integer.toString(i + 1));
            }
            toMove = toMove.otherColor();
        }
        m_variationShown = true;
    }

    public void updateFromGoBoard()
    {
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
            updateFromGoBoard(m_board.getPoint(i));
        //m_lastMove=m_board.getLastMove();
    }

    public void updateFromGoBoard(go.Point point)
    {
        go.Color color = m_board.getColor(point);
        //go.Color.EMPTY
        m_hexBoard.setValue(point.getX(), point.getY(),
            (color==go.Color.BLACK)?1:
            (color==go.Color.WHITE)?2:0);
    }

    private boolean m_needsReset;

    //private boolean m_showCursor = true;

    private boolean m_variationShown;

    private go.Point m_lastMove;

    private go.GameBoard m_board;

    private static final AlphaComposite m_composite3
        = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.3f);

    private Dimension m_minimumFieldSize;

    private Dimension m_preferredFieldSize;

    private HexBoard m_hexBoard;


    private JPanel m_grid;

    private Listener m_listener;

    private void addColumnLabels(int size, int y)
    {
    }

    private void addRowLabel(int x, int y, String text)
    {
    }

    private void calcScore()
    {
        m_board.calcScore();
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            go.Color c = m_board.getScore(p);
            setTerritory(p, c);
        }
    }

    private void clearLastMove()
    {
        m_lastMove = null;
        repaint();
    }



    private void setColor(go.Point p, go.Color color)
    {
        //getField(p).setColor(color); FIXED
        m_hexBoard.setValue(p.getX(), p.getY(),
           (color==go.Color.BLACK)?1:
           (color==go.Color.WHITE)?2:0);
    }


    private void keyPressed(KeyEvent event)
    {
    }

    public Dimension getPreferredSize() {
      return new Dimension(640,480);
    }

    private void setPreferredFieldSize()
    {
        int size = (int)((double)GuiUtils.getDefaultMonoFontSize() * 2.2);
        if (size % 2 == 0)
            ++size;
        m_preferredFieldSize = new Dimension(size, size);
        size = GuiUtils.getDefaultMonoFontSize();
        if (size % 2 == 0)
            ++size;
        m_minimumFieldSize = new Dimension(size, size);
    }

    // **************************************************************
    // **************************************************************
    // ************************************************************** PaintComponent

    final double scale=1.0; // UNUSED


    public void paintComponent(Graphics g)
    {
        // ***************************** get last move:
        int lastx=-1, lasty=-1;
        if (m_lastMove!=null) {
          lastx=m_lastMove.getX();
          lasty=m_lastMove.getY();
        }
        // ***************************** get edgeLength:
        int n=m_hexBoard.getSize(); // Hex Default = 11

        //g.clearRect(0, 0, getWidth(), getHeight());
        g.setColor(new Color(200,200,200));
        g.fillRect(0, 0, getWidth(), getHeight());

        double cos30=0.866; //1/cos30=1.1547
        int w2=getWidth()-4-10;
        int h2=getHeight()-4;
        int polygonRadius = Math.min( (int) ((double)w2 / (2*2*(n-1)*cos30*cos30+2)),
                                      (int) ((double)h2 / n / 2 / cos30));

        // Polygon fr ein Feld anlegen
        //int polygonRadius = Math.min( getWidth() / (int) (2.2 * edgeLength),
        //                             getHeight() / (int) (1.3 * edgeLength)) / 2;
        Polygon polygon = new Polygon();
        Polygon polygon2 = new Polygon();

        Polygon polybreak = new Polygon(); //PHIL

        for(int vertex = 0; vertex < 6; vertex++){
            polygon.addPoint((int) (polygonRadius * Math.cos(vertex * 2 * Math.PI / 6)), (int) (polygonRadius * Math.sin(vertex * 2 * Math.PI / 6)));
            polygon2.addPoint((int) ((6+polygonRadius) * Math.cos(vertex * 2 * Math.PI / 6)),  (int) ((6+polygonRadius) * Math.sin(vertex * 2 * Math.PI / 6)));
            polybreak.addPoint((int) ((6+polygonRadius*0.6) * Math.cos(vertex * 2 * Math.PI / 6 + Math.PI/6)),  (int) ((6+polygonRadius*0.6) * Math.sin(vertex * 2 * Math.PI / 6 + Math.PI/6)));
        }

        // ** RAND ZEICHNEN:
        for (int x = 0; x < n; x++) {
            for (int y = 0; y < n; y++) {
                boolean draw=false;
                if ((y==0 || y==n-1) && (x>0 && x<n-1)) { // Black = top to bottom
                    g.setColor(Color.red); draw=true;   //PHIL
                }
                else if ((x==0 || x==n-1) && (y>0 && y<n-1)) {
                    g.setColor(Color.blue); draw=true;
                }

                if (draw) {
                    int translateX = getWidth() / 2 + (x - y) * (int) ((2 - Math.cos(2 * Math.PI / 6)) * polygonRadius)+10;
                    //int translateY = getHeight() / 2 + (2 * n - 2 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);
                    int translateY = getHeight() / 2 + (n - 1 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);
                    polygon2.translate(translateX, translateY);

                    g.fillPolygon(polygon2);
                    polygon2.translate(-translateX, -translateY);
                }
            }
        }
        // alle Felder zeichnen
        //for(int x = 0; x < edgeLength * 2 - 1; x++) {
            //for(int y = Math.max(x - edgeLength + 1, 0); y < edgeLength * 2 - 1 - Math.max(0, edgeLength - 1 - x); y++){
        for (int x = -1; x < n; x++) { // ==x2 //x==-1 added (for text)
            for (int y = 0; y < n+1; y++) { // ==y2 invers //y==n added (for text)
                int x2=x;
                int y2=(n-1)-y;

                // das Polygon an die entsprechende Stelle schieben
                int translateX = getWidth() / 2 + (x - y) * (int) ((2 - Math.cos(2 * Math.PI / 6)) * polygonRadius)+ 10;
                //int translateY = getHeight() / 2 + (2 * n - 2 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);
                int translateY = getHeight() / 2 + (n - 1 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);

                g.setColor(Color.black);
                if (y==n) {
                  if (x==-1) continue;
                  g.drawString(""+((char)('A'+x2)), translateX, translateY);
                  continue;
                }
                if (x==-1) {
                  translateX+=1;
                  translateY+=7;
                  g.drawString(""+(1+y2), translateX, translateY);
                  continue;
                }
                polygon.translate(translateX, translateY);

                // das Polygon zeichen
                int fieldval=m_hexBoard.getValue(x2, y2);


                switch (fieldval) {
                    //case HexBoard.BLACK: g.setColor(Color.black); break;
                    //case HexBoard.WHITE: g.setColor(Color.white); break; // PHIL couleurs ici
                case HexBoard.BLACK: g.setColor(ColorBlack); break;
                case HexBoard.WHITE: g.setColor(ColorWhite); break; // PHIL couleurs ici
                default: {
                    //g.setColor(new Color(10,192,194));
                    g.setColor(new Color(170,170,170));
                    break;
                }
                }

                g.fillPolygon(polygon);
                //g.setColor(Color.black);
                //g.drawPolygon(polygon);

//                if ((DrawLoose == 1 && fieldval == HexBoard.WHITE) ||
//                    (DrawLoose == 2 && fieldval == HexBoard.BLACK)) {
//                    polybreak.translate(translateX, translateY);
//                    g.setColor(new Color(170,170,170));
//                    g.fillPolygon(polybreak);
//                    polybreak.translate(-translateX, -translateY);
//                    //g.setColor(Color.black);
//                    //g.drawPolygon(polybreak);
//                }

                if ((DrawLoose == 2 && fieldval == HexBoard.WHITE) ||
                    (DrawLoose == 1 && fieldval == HexBoard.BLACK)) {
                    g.drawImage(img1, translateX-polygonRadius/2, translateY-polygonRadius/2,
                                polygonRadius, polygonRadius, this);
                    //g.finalize();
                }


                // draw last move:
                if ((lastx>=0)&&(lasty>=0)&&(x2==lastx)&&(y2==lasty)) // only regular moves
                    //&& f.getLastMove().equals(position))
                {
                    g.setColor(Color.red);
                    g.fillOval(translateX-2, translateY-2, 5, 5);
                    //g.drawOval(translateX-3, translateY-3, 7, 7);
                }

                //if ((f!=null)&&(f.getValue(x, y) == HexBoard.BLACK)) {
                // draw a shadow
                g.setColor(Color.black);
                if (fieldval==HexBoard.BLACK) {
                    g.setColor(Color.gray);
                }
                g.drawPolygon(polygon);

                g.setColor(Color.black); // reset color

                // das Polygon wieder zurckverschieben
                polygon.translate(-translateX, -translateY);

            }
        }
    } // END-METHOD paintComponent

    // **************************************************************
    // **************************************************************
    // ************************************************************** MouseListener's

    public void mousePressed(MouseEvent e) {
        this.requestFocus();
        int width1=(int)((double)getWidth()*scale);
        int height1=(int)((double)getHeight()*scale);

        int n=m_hexBoard.getSize(); // Hex Default = 11
        double cos30=0.866; //1/cos30=1.1547
        int w2=getWidth()-4-10;
        int h2=getHeight()-4;
        int polygonRadius = Math.min( (int) ((double)w2 / (2*2*(n-1)*cos30*cos30+2)),
                                      (int) ((double)h2 / n / 2 / cos30));

        // Polygon fr ein Feld anlegen
        //int polygonRadius = Math.min(width1 / (int) (1.5 * edgeLength), height1 / (int) (1.7 * edgeLength)) / 2;
        Polygon polygon = new Polygon();
        for(int vertex = 0; vertex < 6; vertex++){
            polygon.addPoint((int) (polygonRadius * Math.cos(vertex * 2 * Math.PI / 6)), (int) (polygonRadius * Math.sin(vertex * 2 * Math.PI / 6)));
        }

        // alle Felder prfen
        //for(int x = 0; x < edgeLength * 2 - 1; x++){
            //for(int y = Math.max(x - edgeLength + 1, 0); y < edgeLength * 2 - 1 - Math.max(0, edgeLength - 1 - x); y++){
        for (int x = 0; x < n; x++) {
            for (int y = 0; y < n; y++) {
            //for (int y = Math.max(x - edgeLength + 1, 0); y < edgeLength * 2 - 1 - Math.max(0, edgeLength - 1 - x); y++){
                // das Polygon an die entsprechende Stelle schieben
                int translateX = width1 / 2 + (x - y) * (int) ((2 - Math.cos(2 * Math.PI / 6)) * polygonRadius)+10;
                //int translateY = height1 / 2 + (2 * n - 2 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);
                int translateY = getHeight() / 2 + (n - 1 - (x + y)) * (int) (polygon.getBounds().getSize().height / 2);
                polygon.translate(translateX, translateY);

                // das Polygon prfen
                if (polygon.contains(e.getPoint())){  // fieldClicked?
                    // translate to internal coordinates:
                    int x2=x;
                    int y2=(n-1)-y;
                    fieldClicked(m_board.getPoint(x2, y2), false); // is correct
                    return;
                }

                // das Polygon wieder zurckverschieben
                polygon.translate(-translateX, -translateY);
            }
        }
    }

    public void mouseClicked(MouseEvent e){
    }

    public void mouseReleased(MouseEvent e){
    }

    public void mouseEntered(MouseEvent e){
    }

    public void mouseExited(MouseEvent e){
    }


} // END-CLASS
