//----------------------------------------------------------------------------
// $Id: HGui.java,v 1.425 2004/07/09 17:31:08 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/gui/HGui.java,v $
//----------------------------------------------------------------------------

package gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import game.*;
import go.*;
import gtp.*;
import utils.*;
import version.*;

//----------------------------------------------------------------------------

public class HGui
    extends JFrame
    implements ActionListener, AnalyzeDialog.Callback, GuiBoard.Listener,
               GameTreeViewer.Listener, GtpShell.Callback
{
    HGui(String program, Preferences prefs, String file, int move,
          String time, boolean verbose, boolean computerBlack,
          boolean computerWhite, boolean auto, String gtpFile,
          String gtpCommand, String initAnalyze)
        throws Gtp.Error
    {
        m_prefs = prefs;
        setPrefsDefaults(m_prefs);
        m_boardSize = prefs.getInt("boardsize");
        m_beepAfterMove = prefs.getBool("beep-after-move");
        m_file = file;
        m_gtpFile = gtpFile;
        m_gtpCommand = gtpCommand;
        m_move = move;
        m_computerBlack = computerBlack;
        m_computerWhite = computerWhite;
        m_auto = auto;
        m_verbose = verbose;
        m_initAnalyze = initAnalyze;

        Container contentPane = getContentPane();
        JPanel innerPanel = new JPanel(new BorderLayout());
        contentPane.add(innerPanel, BorderLayout.CENTER);
        m_toolBar = new ToolBar(this);
        contentPane.add(m_toolBar, BorderLayout.NORTH);

        m_infoPanel = new JPanel(new BorderLayout());
        m_timeControl = new TimeControl();
        m_gameInfo = new GameInfo(m_timeControl);
        m_gameInfo.setBorder(utils.GuiUtils.createSmallEmptyBorder());
        m_infoPanel.add(m_gameInfo, BorderLayout.NORTH);

        m_board = new go.GameBoard(m_boardSize);

        m_guiBoard = new GuiBoard(m_board);
        m_guiBoard.setListener(this);
        innerPanel.add(createStatusBar(), BorderLayout.SOUTH);

        //m_squareLayout = new SquareLayout();
        //m_squareLayout.setPreferMultipleOf(2 + 2 * m_boardSize);
        //m_boardPanel = new JPanel(/*m_squareLayout*/);
        //m_boardPanel.add(m_guiBoard);

        Comment.Listener commentListener = new Comment.Listener()
            { public void changed() { cbCommentChanged(); } };
        m_comment = new Comment(commentListener);
        m_infoPanel.add(m_comment, BorderLayout.CENTER);
        m_splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
            m_guiBoard/*m_boardPanel*/, m_infoPanel);
        int condition = JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT;
        InputMap splitPaneInputMap = m_splitPane.getInputMap(condition);
        // According to the docs, null should remove the action,
        // but it does not seem to work with Sun Java 1.4.2, new Object() works
        splitPaneInputMap.put(KeyStroke.getKeyStroke("F8"), new Object());
        m_splitPane.setResizeWeight(0.85);
        innerPanel.add(m_splitPane, BorderLayout.CENTER);

        WindowAdapter windowAdapter = new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
                {
                    close();
                }
            };
        addWindowListener(windowAdapter);
        setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        GuiUtils.setGoIcon(this);

        m_menuBar = new MenuBar(this);
        m_menuBar.selectBoardSizeItem(m_boardSize);
        m_menuBar.selectLevelItem(0);
        m_menuBar.setBeepAfterMove(m_beepAfterMove);
        m_menuBar.setShowLastMove(m_prefs.getBool("show-last-move"));
        m_showLastMove = m_prefs.getBool("show-last-move");
        //m_menuBar.setShowCursor(m_prefs.getBool("show-cursor"));
        //m_guiBoard.setShowCursor(m_prefs.getBool("show-cursor"));
        setJMenuBar(m_menuBar.getMenuBar());
        if (program != null)
            m_program = program;
        else if (m_prefs.contains("program"))
            m_program = m_prefs.getString("program");
        if (m_prefs.contains("program2"))
            m_program2 = m_prefs.getString("program2");
        if (m_program != null && m_program.trim().equals("")) m_program = null;
        if (m_program2 != null && m_program2.trim().equals("")) m_program2 = null;

        m_toolBar.setComputerEnabled(m_program!=null,m_program2!=null);
        m_menuBar.setComputerEnabled(m_program!=null,m_program2!=null);
        m_menuBar.setNormalMode();

        pack();
        m_guiBoard.requestFocusInWindow();
        setTitle(Version.AppName());
        restoreMainWindow();
        try
        {
            if (time != null)
                m_timeControl.setTime(time);
        }
        catch (TimeControl.Error e)
        {
            showWarning(e.getMessage());
        }
        Runnable callback = new Runnable()
            {
                public void run() { initialize(); }
            };
        SwingUtilities.invokeLater(callback);
    }

    public void actionPerformed(ActionEvent event)
    {
        String command = event.getActionCommand();
        if (m_commandInProgress!=0
            && ! command.equals("about")
            && ! command.equals("beep-after-move")
            && ! command.equals("computer-black")
            && ! command.equals("computer-both")
            && ! command.equals("computer-none")
            && ! command.equals("computer-white")
            && ! command.equals("gtp-shell")
            && ! command.equals("help")
            && ! command.equals("interrupt")
            && ! command.equals("show-last-move")
            && ! command.equals("exit"))
            return;
        if (command.equals("about"))
            cbShowAbout();
        else if (command.equals("analyze"))
            cbAnalyze();
        else if (command.equals("all-to-front"))
            cbAllToFront();
        else if (command.equals("attach-program"))
            cbAttachProgram(0);
        else if (command.equals("attach-program2"))
            cbAttachProgram(1);
        else if (command.equals("reattach-program")) {
            cbDetachProgram(0);
            cbAttachProgram(0);
        }
        else if (command.equals("reattach-program2")) {
            cbDetachProgram(1);
            cbAttachProgram(1);
        }
        else if (command.equals("reattach-program3")) { //both
            cbDetachProgram(0);
            cbDetachProgram(1);
            cbAttachProgram(0);
            cbAttachProgram(1);
        }
        else if (command.equals("detach-program"))
            cbDetachProgram(0);
        else if (command.equals("detach-program2"))
            cbDetachProgram(1);
        else if (command.equals("back-to-main-variation"))
            cbBackToMainVar();
        else if (command.equals("backward"))
            cbBackward(1);
        else if (command.equals("backward-10"))
            cbBackward(10);
        else if (command.equals("beep-after-move"))
            cbBeepAfterMove();
        else if (command.equals("beginning"))
            cbBeginning();
        else if (command.equals("board-size-other"))
            cbBoardSizeOther();
        else if (command.startsWith("board-size-"))
            cbNewGame(command.substring("board-size-".length()));
        else if (command.startsWith("level-0")) cbLevel(0);
        else if (command.startsWith("level-1")) cbLevel(1);
        else if (command.startsWith("level-2")) cbLevel(2);
        else if (command.startsWith("level-3")) cbLevel(3);
        else if (command.equals("computer-black"))
            computerBlack();
        else if (command.equals("computer-white"))
            computerWhite();
        else if (command.equals("computer-none"))
            computerNone();
        else if (command.equals("computer-both"))
            cbComputerBoth();
        else if (command.equals("2nd-white"))
            computer2Col(WHITE);
        else if (command.equals("2nd-black"))
            computer2Col(BLACK);
        else if (command.equals("enter-moves"))
            cbEnterMoves();
        else if (command.equals("enter-moves2"))
            cbEnterMoves2();
        else if (command.equals("end"))
            cbEnd();
        else if (command.equals("exit"))
            close();
        else if (command.equals("forward"))
            cbForward(1);
        else if (command.equals("forward-10"))
            cbForward(10);
        else if (command.equals("game-info"))
            cbGameInfo();
        else if (command.equals("goto"))
            cbGoto();
        else if (command.equals("gtp-shell"))
            cbGtpShell();
        //else if (command.startsWith("handicap-"))
        //    cbHandicap(command.substring("handicap-".length()));
        else if (command.equals("help"))
            cbHelp();
        else if (command.equals("interrupt"))
            cbInterrupt();
        else if (command.equals("keep-only-main-variation"))
            cbKeepOnlyMainVariation();
        else if (command.equals("keep-only-position"))
            cbKeepOnlyPosition();
        else if (command.equals("make-main-variation"))
            cbMakeMainVariation();
        else if (command.equals("next-variation"))
            cbNextVariation();
        else if (command.equals("new-game"))
            cbNewGame(m_boardSize);
        else if (command.equals("open"))
            cbOpen();
        else if (command.equals("open-recent"))
            cbOpenRecent();
        else if (command.equals("swap"))
            cbSwap();
        else if (command.equals("play"))
            cbPlay();
        else if (command.equals("previous-variation"))
            cbPreviousVariation();
        else if (command.equals("print"))
            cbPrint();
        else if (command.equals("save"))
            cbSaveAs();
        else if (command.equals("save1")) // normal save
            cbSave();
        else if (command.equals("save-auto"))
            cbSaveAutoFilename(true);
        else if (command.equals("save-position"))
            cbSavePosition();
        else if (command.equals("setup"))
            cbSetup();
        else if (command.equals("setup-black"))
            cbSetupBlack();
        else if (command.equals("setup-white"))
            cbSetupWhite();
        //else if (command.equals("show-cursor"))
        //    cbShowCursor();
        else if (command.equals("show-gametree"))
            cbShowGameTree();
        else if (command.equals("show-last-move"))
            cbShowLastMove();
        else if (command.equals("truncate"))
            cbTruncate();
        else
            assert(false);
    }

    public void cbAnalyze()
    {
        if (m_commandThread == null)
            return;
        if (m_analyzeDialog == null)
        {
            m_analyzeDialog =
                new AnalyzeDialog(this, m_prefs,
                                  m_commandThread.getSupportedCommands(),
                                  m_commandThread);
            restoreSize(m_analyzeDialog, "window-analyze", m_boardSize);
            setTitle();
        }
        m_analyzeDialog.toTop();
    }

    public void cbAllToFront()
    {
        if (m_help != null && m_help.isVisible())
            m_help.toTop();
        if (m_gtpShell != null && m_gtpShell.isVisible())
            m_gtpShell.toTop();
        if (m_gtpShell2 != null && m_gtpShell2.isVisible())
            m_gtpShell2.toTop();
        if (m_analyzeDialog != null && m_analyzeDialog.isVisible())
            m_analyzeDialog.toTop();
        if (m_gameTreeViewer != null && m_gameTreeViewer.isVisible())
            m_gameTreeViewer.toTop();
        toTop();
    }

    public void cbAttachProgram(int i)
    {
        if (i==0 && m_commandThread != null) return;
        if (i==1 && m_commandThread2 != null) return;
        fixAttachExeFilename(m_boardSize);
        String program = SelectProgram.select(this, i);
        if (program == null) return;
        if (m_boardSize>15 && (-1!=program.indexOf("six_gtp_engine"))) {
          //showInfo("Warning: Board is larger than 15: \"six_gtp_engine.exe\" cannot play on such large boards.");
          showInfo("Warning: Board is larger than 15: \"six\" engine does not support this.");
        }
        int level=getLevelFromString(program);
        m_menuBar.selectLevelItem(level);
        if (attachProgram(i, program))
            m_prefs.setString("program"+(i==0?"":"2"), (i==0?m_program:m_program2));
        else
            m_prefs.setString("program"+(i==0?"":"2"), "");
    }

    public void cbDetachProgram(int i)
    {
        if (i==0 && m_commandThread == null) return;
        if (i==1 && m_commandThread2 == null) return;
        if (m_commandInProgress!=0)
            if (! showQuestion("Kill program?"))
                return;
        detachProgram(i);
        m_prefs.setString("program"+(i==1?"2":""), "");
    }

    public void cbGtpShell()
    {
        if (m_gtpShell != null)
            m_gtpShell.toTop();
        if (m_gtpShell2 != null)
            m_gtpShell2.toTop();
    }

    public void cbShowGameTree()
    {
        if (m_gameTreeViewer == null)
        {
            m_gameTreeViewer = new GameTreeViewer(this);
            restoreSize(m_gameTreeViewer, "window-gametree", m_boardSize);
        }
        m_gameTreeViewer.update(m_gameTree, m_currentNode);
        m_gameTreeViewer.toTop();
    }

    public void clearAnalyzeCommand()
    {
        if (m_analyzeCommand == null)
            return;
        if (m_commandInProgress!=0)
        {
            showError("Cannot clear analyze command\n" +
                      "while command in progress.");
            return;
        }
        if (m_setupMode)
        {
            showError("Cannot clear analyze command\n" +
                      "in setup mode.");
            return;
        }
        m_analyzeCommand = null;
        setBoardCursorDefault();
        resetBoard();
        clearStatus();
    }

    public void fieldClicked(go.Point p, boolean modifiedSelect)
    {
        if (m_commandInProgress!=0)
            return;
        if (m_setupMode && ! modifiedSelect)
        {
            if (m_board.getColor(p) == go.Color.EMPTY)
                m_board.play(new Move(p, m_setupColor));
            else if (m_board.getColor(p) == m_setupColor)
                m_board.play(new Move(p, m_setupColor.otherColor()));
            else
                m_board.play(new Move(p, go.Color.EMPTY));
            m_board.setToMove(m_setupColor);
            updateGameInfo(true);
            m_guiBoard.updateFromGoBoard();
            m_guiBoard.repaint();
            m_needsSave = true;
        }
        else if (m_analyzeCommand != null && m_analyzeCommand.needsPointArg()
                 && ! modifiedSelect)
        {
            m_analyzeCommand.setPointArg(p);
            m_guiBoard.clearAllSelect();
            m_guiBoard.setSelect(p, true);
            m_guiBoard.repaint();
            analyzeBegin(false, false);
            return;
        }
        else if (m_analyzeCommand != null
                 && m_analyzeCommand.needsPointListArg())
        {
            Vector pointListArg = m_analyzeCommand.getPointListArg();
            if (pointListArg.contains(p))
            {
                pointListArg.remove(p);
                if (modifiedSelect)
                    pointListArg.add(p);
            }
            else
                pointListArg.add(p);
            m_guiBoard.clearAllSelect();
            for (int i = 0; i < pointListArg.size(); ++i)
                m_guiBoard.setSelect((go.Point)pointListArg.get(i), true);
            m_guiBoard.repaint();
            if (modifiedSelect && pointListArg.size() > 0)
                analyzeBegin(false, false);
            return;
        }
        else if (m_scoreMode && ! modifiedSelect)
        {
            m_guiBoard.scoreSetDead(p);
            m_guiBoard.repaint();
            float komi = m_gameTree.getGameInformation().m_komi;
            m_scoreDialog.showScore(m_board.scoreGet(komi, getRules()));
            return;
        }
        else if (! modifiedSelect)
        {
            Move move = new Move(p, m_board.getToMove());
            humanMoved(move);
        }
    }


    public void gotoNode(Node node)
    {
        // GameTreeViewer is not disabled in score mode
        if (m_scoreMode)
            return;
        Vector shortestPath = m_currentNode.getShortestPath(node);
        for (int i = 0; i < shortestPath.size(); ++i)
        {
            Node nextNode = (Node)shortestPath.get(i);
            if (nextNode == m_currentNode)
            {
                if (! backward(1))
                    break;
            }
            else
            {
                if (! checkCurrentNodeExecuted())
                    break;
                assert(nextNode.isChildOf(m_currentNode));
                m_currentNode = nextNode;
                try
                {
                    executeCurrentNode();
                }
                catch (Gtp.Error e)
                {
                    showGtpError(e);
                    break;
                }
            }
        }
        boardChangedBegin(false, false);
    }

    public static boolean OPTION_2nd_gtp=false;
    public static void main(String[] args)
    {
        try
        {
            String lookAndFeel = UIManager.getSystemLookAndFeelClassName();
            UIManager.setLookAndFeel(lookAndFeel);
        }
        catch (Exception e)
        {
        }
        boolean verbose = false;
        try
        {
            String options[] = {
                "analyze:",
                "auto",
                "command:",
                "computer-black",
                "computer-both",
                "computer-none",
                "config:",
                "file:",
                "gtpfile:",
                "help",
                "komi:",
                "move:",
                "rules:",
                "size:",
                "time:",
                "verbose",
                "version",
                "2" // 2nd GTP
            };
            Options opt = new Options(args, options);
            opt.handleConfigOption();
            if (opt.isSet("help"))
            {
                String helpText =
                    "Usage: java -jar gogui.jar [options] program\n" +
                    "Graphical user interface for Go programs\n" +
                    "using the Go Text Protocol.\n" +
                    "\n" +
                    "-analyze name   initialize analyze command\n" +
                    "-auto           auto play games (if computer both)\n" +
                    "-command cmd    send GTP command at startup\n" +
                    "-computer-both  computer plays both sides\n" +
                    "-computer-black computer plays black\n" +
                    "-computer-none  computer plays no side\n" +
                    "-config         config file\n" +
                    "-file filename  load SGF file\n" +
                    "-gtpfile file   send GTP file at startup\n" +
                    "-help           display this help and exit\n" +
                    "-komi value     set komi\n" +
                    "-move n         load SGF file until move number\n" +
                    "-rules name     use rules (chinese|japanese)\n" +
                    "-size n         set board size\n" +
                    "-time spec      set time limits (min[+min/moves])\n" +
                    "-verbose        print debugging messages\n" +
                    "-version        print version and exit\n" +
                    "-2              activate feature to attach 2nd gtp program (plays black)\n";
                System.out.print(helpText);
                System.exit(0);
            }
            if (opt.isSet("version"))
            {
                System.out.println(Version.AppName()+" " + Version.getVersion());
                System.exit(0);
            }
            if (opt.isSet("2")) {
              System.out.println("2nd GTP activated.");
              OPTION_2nd_gtp=true;
            } else {
              //System.out.println("2nd GTP not activated.");
            }
            Preferences prefs = new Preferences();
            String initAnalyze = opt.getString("analyze");
            boolean auto = opt.isSet("auto");
            boolean computerBlack = false;
            boolean computerWhite = true;
            if (opt.isSet("computer-none"))
                computerWhite = false;
            else if (opt.isSet("computer-black"))
            {
                computerBlack = true;
                computerWhite = false;
            }
            else if (opt.isSet("computer-both"))
                computerBlack = true;
            //String file = opt.getString("file", "");
            String gtpFile = opt.getString("gtpfile", "");
            String gtpCommand = opt.getString("command", "");
            if (opt.contains("komi"))
                prefs.setFloat("komi", opt.getFloat("komi"));
            int move = opt.getInteger("move", -1);
            if (opt.contains("size"))
                prefs.setInt("boardsize", opt.getInteger("size"));
            String rules = opt.getString("rules", "");
            prefs.setString("rules", rules);
            String time = opt.getString("time", null);
            verbose = opt.isSet("verbose");
            Vector arguments = opt.getArguments();
            /*
            String program = null;
            if (arguments.size() == 1)
                program = (String)arguments.get(0);
            else if (arguments.size() > 1)
                throw new Exception("Only one program argument allowed.");
            */
            // load sgf file (instead of program argument):
            String file = opt.getString("file", "");
            if (file.equals("") && arguments.size() == 1)
                file = (String)arguments.get(0);
            else if (arguments.size() > 1 || ((!file.equals("") && arguments.size() == 1)))
                throw new Exception("Only one sgf file argument allowed.");
            // 1st arg of c'tor: "program" is null now!
            new HGui(null, prefs, file, move, time, verbose, computerBlack,
                      computerWhite, auto, gtpFile, gtpCommand, initAnalyze);
        }
        catch (AssertionError e)
        {
            SimpleDialogs.showError(null, "Assertion error");
            e.printStackTrace();
            System.exit(-1);
        }
        catch (RuntimeException e)
        {
            SimpleDialogs.showError(null, StringUtils.formatException(e));
            e.printStackTrace();
            System.exit(-1);
        }
        catch (Throwable t)
        {
            SimpleDialogs.showError(null, StringUtils.formatException(t));
            t.printStackTrace();
            System.exit(-1);
        }
    }

    public boolean sendGtpCommand(String command, boolean sync)
        throws Gtp.Error
    {
        if (m_commandInProgress!=0 || m_commandThread == null)
            return false;
        if (sync)
        {
            m_commandThread.sendCommand(command);
            return true;
        }
        Runnable callback = new Runnable()
            {
                public void run() { sendGtpCommandContinue(); }
            };
        beginLengthyCommand(0); //#
        m_commandThread.sendCommand(command, callback);
        return true;
    }

    public void sendGtpCommandContinue()
    {
        m_commandThread.getException();
        endLengthyCommand(0);
    }

    public void initAnalyzeCommand(AnalyzeCommand command, boolean autoRun)
    {
        if (m_commandThread == null)
            return;
        m_analyzeCommand = command;
        m_analyzeAutoRun = autoRun;
        if (command.needsPointArg())
        {
            setBoardCursor(Cursor.CROSSHAIR_CURSOR);
            showStatusSelectTarget();
        }
        else if (command.needsPointListArg())
        {
            setBoardCursor(Cursor.CROSSHAIR_CURSOR);
            showStatusSelectPointList();
        }
    }

    public void setAnalyzeCommand(AnalyzeCommand command, boolean autoRun,
                                  boolean clearBoard)
    {
        if (m_commandInProgress!=0)
        {
            showError("Cannot run analyze command\n" +
                      "while command in progress.");
            return;
        }
        if (m_setupMode)
        {
            showError("Cannot run analyze command\n" +
                      "in setup mode.");
            return;
        }
        initAnalyzeCommand(command, autoRun);
        if (m_analyzeCommand.needsPointArg()
            || m_analyzeCommand.needsPointListArg())
        {
            m_guiBoard.clearAllSelect();
            m_guiBoard.repaint();
            toTop();
        }
        else
            analyzeBegin(false, clearBoard);
    }

    public void toTop()
    {
        setState(Frame.NORMAL);
        toFront();
    }

    private class AnalyzeContinue
        implements Runnable
    {
        public AnalyzeContinue(boolean checkComputerMove,
                               boolean resetBoard)
        {
            m_checkComputerMove = checkComputerMove;
            m_resetBoard = resetBoard;
        }

        public void run()
        {
            analyzeContinue(m_checkComputerMove, m_resetBoard);
        }

        private boolean m_checkComputerMove;

        private boolean m_resetBoard;
    }

    private boolean m_analyzeAutoRun;

    private boolean m_auto;

    private boolean m_beepAfterMove;

    private boolean m_computerBlack;

    private boolean m_computerWhite;

    private int     m_commandInProgress; // 0 or 1 or 2 (or 3=both, does not happen)

    private boolean m_isRootExecuted;

    private boolean m_lostOnTimeShown;

    private boolean m_needsSave;

    //private boolean m_resigned;

    private boolean m_scoreMode;

    private boolean m_setupMode;

    private boolean m_showLastMove;

    private boolean m_verbose;

    private int m_boardSize;

    private int m_currentNodeExecuted;

    private int m_handicap;

    private int m_move;

    private go.GameBoard m_board;

    private go.Color m_setupColor;

    private GuiBoard m_guiBoard;

    private CommandThread m_commandThread;
    private CommandThread m_commandThread2;

    private Comment m_comment;

    private File m_loadedFile;

    private GameInfo m_gameInfo;

    private GtpShell m_gtpShell;
    private GtpShell m_gtpShell2;

    private GameTree m_gameTree;

    private GameTreeViewer m_gameTreeViewer;

    private Help m_help;

    private JLabel m_statusLabel;

    //private JPanel m_boardPanel;

    private JPanel m_infoPanel;

    private JSplitPane m_splitPane;

    private MenuBar m_menuBar;

    private Node m_currentNode;

    private AnalyzeCommand m_analyzeCommand;

    private String m_file;

    private String m_gtpCommand;

    private String m_gtpFile;

    private String m_initAnalyze;

    private String m_name = "";
    private String m_name2 = "";

    private String m_program;
    private String m_program2;

    private String m_version = "";
    private String m_version2 = "";

    private AnalyzeDialog m_analyzeDialog;

    private Preferences m_prefs;

    private ScoreDialog m_scoreDialog;

    //private SquareLayout m_squareLayout;

    private TimeControl m_timeControl;

    private ToolBar m_toolBar;

    static
    {
        // Must be set before static initializers of apple.awt are executed.
        System.setProperty("apple.awt.brushMetalLook", "true");
    }

    private void addPlayerComputerToGameInfo(go.Color color, int idx)
    {
        GameInformation gameInformation = m_gameTree.getGameInformation();
        String name;
        if (idx==0) {
          if (m_commandThread == null) return;
          name = m_name;
          if (m_version != null && ! m_version.equals(""))
              name = name +  ":" + m_version;
        } else {
          if (m_commandThread2 == null) return;
          name = m_name2;
          if (m_version2 != null && ! m_version2.equals(""))
              name = name +  ":" + m_version2;
        }
        if (color == go.Color.BLACK
            && (gameInformation.m_playerBlack == null
                || gameInformation.m_playerBlack.trim().equals("")
                || gameInformation.m_playerBlack.toUpperCase().indexOf("GTP")!=-1
            ))
        {
          gameInformation.m_playerBlack = name;
        }
        else if (color == go.Color.WHITE
            && (gameInformation.m_playerWhite == null
                || gameInformation.m_playerWhite.trim().equals("")
                || gameInformation.m_playerWhite.toUpperCase().indexOf("GTP")!=-1
            ))
        {
          gameInformation.m_playerWhite = name;
        }
    }

    private void analyzeBegin(boolean checkComputerMove, boolean resetBoard)
    {
        if (m_commandThread == null)
            return;
        if (m_analyzeCommand.isPointArgMissing())
            return;
        showStatus("Running " + m_analyzeCommand.getResultTitle() + " ...");
        go.Color toMove = m_board.getToMove();
        go.Color color = m_analyzeDialog.getSelectedColor();
        String command = m_analyzeCommand.replaceWildCards(toMove, color);
        runLengthyCommand(0,command,
                          new AnalyzeContinue(checkComputerMove, resetBoard));
    }

    private void analyzeContinue(boolean checkComputerMove, boolean resetBoard)
    {
        endLengthyCommand(0);
        String title = m_analyzeCommand.getResultTitle();
        try
        {
            if (resetBoard)
                resetBoard();
            Gtp.Error e = m_commandThread.getException();
            if (e != null)
                throw e;
            String response = m_commandThread.getResponse();
            AnalyzeShow.show(m_analyzeCommand, m_guiBoard, m_board, response);
            int type = m_analyzeCommand.getType();
            boolean statusContainsResponse = false;
            if (type == AnalyzeCommand.STRING
                || type == AnalyzeCommand.HSTRING
                || type == AnalyzeCommand.HPSTRING
                || type == AnalyzeCommand.PSTRING
                || type == AnalyzeCommand.VAR
                || type == AnalyzeCommand.VARC
                || type == AnalyzeCommand.VARW
                || type == AnalyzeCommand.VARB
                || type == AnalyzeCommand.VARP
                || type == AnalyzeCommand.VARPO)
            {
                if (response.indexOf("\n") < 0)
                {
                    showStatus(title + ": " + response);
                    statusContainsResponse = true;
                }
                else
                {
                    boolean highlight =
                        (type == AnalyzeCommand.HSTRING
                         || type == AnalyzeCommand.HPSTRING);
                    AnalyzeTextOutput.Listener listener = null;
                    if (type == AnalyzeCommand.PSTRING
                        || type == AnalyzeCommand.HPSTRING)
                    {
                        listener = new AnalyzeTextOutput.Listener()
                            {
                                public void textSelected(String text)
                                {
                                    go.Point list[] =
                                        Gtp.parsePointString(text,
                                                             m_boardSize);
                                    m_guiBoard.showPointList(list);
                                    m_guiBoard.repaint();
                                }
                            };
                    }
                    new AnalyzeTextOutput(this, title, response, highlight,
                                          listener);
                }
            }
            if (! statusContainsResponse)
                showStatus(title);
            if (checkComputerMove)
                checkComputerMove();
        }
        catch(Gtp.Error e)
        {
            showStatus(title);
            showGtpError(e);
            return;
        }
    }

    /** Attach program.
        @return true if program was successfully attached.
    */
    private boolean attachProgram(int idx, String program)
    {
        if (idx==1) return attachProgram2(program);
        program = program.trim();
        if (program.equals(""))
            return false;
        m_program = program;
        m_gtpShell = new GtpShell("GTP - "+Version.AppName(), this, m_prefs);
        m_gtpShell.setProgramCommand(program);
        try
        {
            Gtp gtp = new Gtp(m_program, m_verbose, m_gtpShell);
            m_commandThread = new CommandThread(gtp);
            m_commandThread.start();
        }
        catch (Gtp.Error e)
        {
            SimpleDialogs.showError(this,
                                    e.getMessage() + "\n"
                                    + "See GTP shell for any error messages\n"
                                    + "printed by the program.");
            m_toolBar.setComputerEnabled(false, m_commandThread2!=null); //false
            m_menuBar.setComputerEnabled(false, m_commandThread2!=null); //false
            return false;
        }
        m_menuBar.setComputerEnabled(true, m_commandThread2!=null);
        m_toolBar.setComputerEnabled(true, m_commandThread2!=null);
        try
        {
            setFastUpdate(true);
            m_name = m_commandThread.sendCommand("name").trim();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
            return false;
        }
        finally
        {
            setFastUpdate(false);
        }
        try
        {
            m_commandThread.queryProtocolVersion();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        try
        {
            m_version = m_commandThread.queryVersion();
            m_gtpShell.setProgramVersion(m_version);
            m_commandThread.querySupportedCommands();
            m_commandThread.queryInterruptSupport();
        }
        catch (Gtp.Error e)
        {
        }
        restoreSize(m_gtpShell, "window-gtpshell", m_boardSize);
        if (m_prefs.getBool("show-gtpshell")) m_gtpShell.toTop();
        if (m_prefs.getBool("show-analyze"))
            cbAnalyze();
        m_gtpShell.setProgramName(m_name);
        Vector supportedCommands =
            m_commandThread.getSupportedCommands();
        m_gtpShell.setInitialCompletions(supportedCommands);
        if (! m_gtpFile.equals(""))
            m_gtpShell.sendGtpFile(new File(m_gtpFile));
        if (! m_gtpCommand.equals(""))
            sendGtpString(m_gtpCommand);
        Node oldCurrentNode = m_currentNode;
        m_board.initSize(m_boardSize);
        executeRoot(); // will set m_currentNode=root
        gotoNode(oldCurrentNode);
        setTitle();
        return true;
    }

    /** Attach program.
        @return true if program was successfully attached.
    */
    private boolean attachProgram2(String program)
    {
        program = program.trim();
        if (program.equals(""))
            return false;
        m_program2 = program;
        m_gtpShell2 = new GtpShell("2nd GTP - "+Version.AppName(), this, m_prefs);
        m_gtpShell2.setProgramCommand(program);
        try
        {
            Gtp gtp = new Gtp(m_program2, m_verbose, m_gtpShell2);
            m_commandThread2 = new CommandThread(gtp);
            m_commandThread2.start();
        }
        catch (Gtp.Error e)
        {
            SimpleDialogs.showError(this,
                                    e.getMessage() + "\n"
                                    + "See GTP shell for any error messages\n"
                                    + "printed by the program.");
            m_toolBar.setComputerEnabled(m_commandThread!=null, false);
            m_menuBar.setComputerEnabled(m_commandThread!=null, false);
            return false;
        }
        m_menuBar.setComputerEnabled(m_commandThread!=null, true);
        m_toolBar.setComputerEnabled(m_commandThread!=null, true);
        try
        {
            setFastUpdate(true);
            m_name2 = m_commandThread2.sendCommand("name").trim();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
            return false;
        }
        finally
        {
            setFastUpdate(false);
        }
        try
        {
            m_commandThread2.queryProtocolVersion();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        try
        {
            m_version2 = m_commandThread2.queryVersion();
            m_gtpShell2.setProgramVersion(m_version2);
            m_commandThread2.querySupportedCommands();
            m_commandThread2.queryInterruptSupport();
        }
        catch (Gtp.Error e)
        {
        }
        restoreSize(m_gtpShell2, "window-gtpshell2", m_boardSize);
        if (m_prefs.getBool("show-gtpshell2")) m_gtpShell2.toTop();
        m_gtpShell2.setProgramName(m_name2);
        Vector supportedCommands =
            m_commandThread2.getSupportedCommands();
        m_gtpShell2.setInitialCompletions(supportedCommands);
        Node oldCurrentNode = m_currentNode;
        m_board.initSize(m_boardSize);
        executeRoot();
        gotoNode(oldCurrentNode);
        setTitle();
        return true;
    }

    // undo till LastMove == m
    // m must be normal move!!
    private void cbBackward2(Move m) {
        int undone=0;
        computerNone();
        setFastUpdate(true);
        try
        {
            while (true) {
                //Node father=m_currentNode.getFather();
                 //if (father == null) return;
                Move mm=m_currentNode.getMove();
                if (mm!=null && m.getPoint().equals(mm.getPoint())) return; //colour does not matter!
                undoCurrentNode(); undone++;
                m_currentNode = m_currentNode.getFather();
                m_currentNodeExecuted = m_currentNode.getAllAsMoves().size();
            }
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        finally
        {
            setFastUpdate(false);
            if (undone>0) boardChangedBegin(false, false);
        }
    }

    private boolean backward(int n)
    {
        setFastUpdate(true);
        try
        {
            for (int i = 0; i < n; ++i)
            {
                if (m_currentNode.getFather() == null)
                    return false;
                undoCurrentNode();
                m_currentNode = m_currentNode.getFather();
                m_currentNodeExecuted = m_currentNode.getAllAsMoves().size();
            }
            computerNone();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
            return false;
        }
        finally
        {
            setFastUpdate(false);
        }
        return true;
    }

    private void beginLengthyCommand(int idx)
    {
        setBoardCursor(Cursor.WAIT_CURSOR);
        boolean isInterruptSupported =
            ((idx==0 && m_commandThread.isInterruptSupported()) ||
             (idx==1 && m_commandThread2.isInterruptSupported()) ||
             (m_computerBlack && m_computerWhite));
        m_menuBar.setCommandInProgress(isInterruptSupported);
        m_toolBar.setCommandInProgress(isInterruptSupported);
        if (idx==0) m_gtpShell.setCommandInProgess(true);
              else m_gtpShell2.setCommandInProgess(true);
        m_commandInProgress = idx+1;
    }

    private void endLengthyCommand(int idx)
    {
        clearStatus();
        m_menuBar.setNormalMode();
        m_toolBar.enableAll(true, m_currentNode);
        if (idx==0) m_gtpShell.setCommandInProgess(false);
              else m_gtpShell2.setCommandInProgess(false);
        m_commandInProgress = 0;
        if (m_analyzeCommand != null
            && (m_analyzeCommand.needsPointArg()
                || m_analyzeCommand.needsPointListArg()))
            setBoardCursor(Cursor.CROSSHAIR_CURSOR);
        else
            setBoardCursorDefault();
    }

    private void boardChangedBegin(boolean doCheckComputerMove,
                                   boolean gameTreeChanged)
    {
        m_guiBoard.updateFromGoBoard();
        updateGameInfo(gameTreeChanged);
        m_toolBar.updateGameButtons(m_currentNode);
        m_menuBar.updateGameMenuItems(m_gameTree, m_currentNode);
        m_menuBar.selectBoardSizeItem(m_board.getSize());
        clearStatus();
        if (m_commandThread != null
            && isCurrentNodeExecuted()
            && m_analyzeCommand != null
            && m_analyzeAutoRun
            && ! m_analyzeCommand.isPointArgMissing())
            analyzeBegin(doCheckComputerMove, true);
        else
        {
            resetBoard();
            if (doCheckComputerMove)
                checkComputerMove();
        }
    }

    private void cbBeepAfterMove()
    {
        m_beepAfterMove = m_menuBar.getBeepAfterMove();
        m_prefs.setBool("beep-after-move", m_beepAfterMove);
    }

    private void cbBeginning()
    {
        backward(m_currentNode.getDepth());
        boardChangedBegin(false, false);
    }

    private void cbBackToMainVar()
    {
        Node node = m_currentNode.getBackToMainVariation();
        gotoNode(node);
    }

    private void cbBackward(int n)
    {
        backward(n);
        boardChangedBegin(false, false);
    }


    //sequence
    private void cbEnterMoves() {
       String s = JOptionPane.showInputDialog(this, "Enter Moves (e.g. D2/E4/S4):", "");
       if (s == null) return; //cancel
       ArrayList<Move> vv=new ArrayList<Move>();
       String[] v = s.split("[\\/, ]+"); // chars "/," or " "+
       if (v==null) return;
       go.Color nextCol=m_board.getToMove();
       if (m_board.isGameOver() || nextCol==go.Color.EMPTY) return;
       try {
           for (int i=0;i<v.length;i++) {
               vv.add(Move.parseMove(nextCol, v[i], m_boardSize, false));
               nextCol=nextCol.otherColor();
           }
       }
       catch (Exception e) {
           return;
       }
       addMovesInternal(vv);
    }

    // as children
    private void cbEnterMoves2() {
       String s = JOptionPane.showInputDialog(this, "Enter Moves (e.g. D2/E4/S4):", "");
       if (s == null) return; //cancel
       ArrayList<Move> vv=new ArrayList<Move>();
       String[] v = s.split("[\\/, ]+"); // chars "/," or " "+
       if (v==null) return;
       go.Color nextCol=m_board.getToMove();
       if (m_board.isGameOver() || nextCol==go.Color.EMPTY) return;
       try {
           for (int i=0;i<v.length;i++) {
               vv.add(Move.parseMove(nextCol, v[i], m_boardSize, false));
           }
       }
       catch (Exception e) {
           return;
       }
       addMovesInternal2Children(vv);
    }

    private void cbBoardSizeOther()
    {
        String value =
            JOptionPane.showInputDialog(this, "Board size",
                                        Integer.toString(m_boardSize));
        if (value == null)
            return;
        int boardSize = -1;
        try
        {
            boardSize = Integer.parseInt(value);
        }
        catch (NumberFormatException e)
        {
        }
        if (boardSize < 4 || boardSize > 26)
        {
            showError("Invalid board size");
            return;
        }
        cbNewGame(boardSize);
    }

    private void cbCommentChanged()
    {
        m_needsSave = true;
        if (m_gameTreeViewer != null)
            m_gameTreeViewer.redrawCurrentNode();
    }

    private void cbComputerBoth()
    {
        computerBoth();
        checkComputerMove();
    }

    private void cbEnd()
    {
        forward(m_currentNode.getNodesLeft());
        boardChangedBegin(false, false);
    }

    private void cbForward(int n)
    {
        forward(n);
        boardChangedBegin(false, false);
    }

    private void cbGameInfo()
    {
        GameInformation gameInformation = m_gameTree.getGameInformation();
        GameInfoDialog.show(this, gameInformation);
        if (gameInformation.m_komi != m_prefs.getFloat("komi"))
        {
            m_prefs.setFloat("komi", gameInformation.m_komi);
            setKomi(gameInformation.m_komi);
        }
        if (gameInformation.m_rules != null
            && ! gameInformation.m_rules.equals(m_prefs.getString("rules")))
        {
            m_prefs.setString("rules", gameInformation.m_rules);
            setRules();
        }
        setTitle();
    }

    private void cbGoto()
    {
        String value = JOptionPane.showInputDialog(this, "Move Number");
        if (value == null || value.equals(""))
            return;
        try
        {
            int gotoNumber = Integer.parseInt(value);
            int moveNumber = m_currentNode.getMoveNumber();
            int movesLeft = m_currentNode.getMovesLeft();
            if (gotoNumber < 0 || gotoNumber > moveNumber + movesLeft)
            {
                showError("Invalid move number");
                return;
            }
            int numberNodes = 0;
            Node node = m_currentNode;
            if (gotoNumber < moveNumber)
            {
                while (node.getFather() != null && moveNumber > gotoNumber)
                {
                    if (node.getMove() != null)
                        --moveNumber;
                    ++numberNodes;
                    node = node.getFather();
                }
                backward(numberNodes);
            }
            else
            {
                while (node.getChild() != null && moveNumber < gotoNumber)
                {
                    if (node.getMove() != null)
                        ++moveNumber;
                    ++numberNodes;
                    node = node.getChild();
                }
                forward(numberNodes);
            }
            boardChangedBegin(false, false);
        }
        catch (NumberFormatException e)
        {
            showError("Invalid move number");
        }
    }

    /*
    private void cbHandicap(String handicap)
    {
        try
        {
            m_handicap = Integer.parseInt(handicap);
            if (m_board.isModified())
                showInfo("Handicap will take effect on next game.");
            else
            {
                computerBlack();
                newGame(m_boardSize);
            }
        }
        catch (NumberFormatException e)
        {
            assert(false);
        }
    }
    */

    private void cbHelp()
    {
        URL u = getClass().getClassLoader().getResource("doc/index.html");
        if (u == null)
        {
            showError("Help not found");
            return;
        }
        if (m_help == null)
        {
            m_help = new Help(u);
            restoreSize(m_help, "window-help", m_boardSize);
        }
        m_help.toTop();
    }

    private void cbInterrupt()
    {
        if (m_commandInProgress==0) return;
        if (m_commandThread != null && !m_commandThread.isProgramDead()) {
          if (m_computerBlack && m_computerWhite) computerNone();
          if (! m_commandThread.isInterruptSupported()) return;
          //if (! showQuestion("Interrupt command?")) return; //#changed: do not ask
          sendInterrupt(0);
        }
        if (m_commandThread2 != null && !m_commandThread2.isProgramDead()) {
          if (m_computerBlack && m_computerWhite) computerNone();
          if (! m_commandThread2.isInterruptSupported()) return;
          //if (! showQuestion("Interrupt command (gtp2)?")) return; //#changed: do not ask
          sendInterrupt(1);
        }
    }

    private void cbKeepOnlyMainVariation()
    {
        if (! m_currentNode.isInMainVariation())
            return;
        if (! showQuestion("Delete all variations except main?"))
            return;
        m_gameTree.keepOnlyMainVariation();
        m_needsSave = true;
        boardChangedBegin(false, true);
    }

    private void cbKeepOnlyPosition()
    {
        if (! showQuestion("Delete all moves?"))
            return;
        GameInformation gameInformation = m_gameTree.getGameInformation();
        m_gameTree = new GameTree(m_boardSize, gameInformation.m_komi, null,
                                  gameInformation.m_rules);
        Node root = m_gameTree.getRoot();
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point point = m_board.getPoint(i);
            go.Color color = m_board.getColor(point);
            if (color == go.Color.BLACK)
                root.addBlack(point);
            else if (color == go.Color.WHITE)
                root.addWhite(point);
        }
        root.setPlayer(m_board.getToMove());
        m_board.initSize(m_boardSize);
        executeRoot();
        m_needsSave = true;
        boardChangedBegin(false, true);
    }

    private void cbMakeMainVariation()
    {
        if (! showQuestion("Make current node to main variation?"))
            return;
        m_currentNode.makeMainVariation();
        m_needsSave = true;
        boardChangedBegin(false, true);
    }

    private void cbNewGame(int size)
    {
        if (m_needsSave && ! checkSaveGame())
            return;
        m_prefs.setInt("boardsize", size);
        fileModified();
        if (((size>11  && m_boardSize<=11)||
            (size<=11 && m_boardSize>11))) { //# for HexGui together with Six
          if (m_commandThread!=null && m_program!=null && -1!=m_program.indexOf("six_gtp_engine")) {
            cbDetachProgram(0);
          }
          if (m_commandThread2!=null && m_program2!=null && -1!=m_program2.indexOf("six_gtp_engine")) {
            cbDetachProgram(1);
          }
        }
        newGame(size);
    }

    private void cbNewGame(String size)
    {
        try
        {
            cbNewGame(Integer.parseInt(size));
        }
        catch (NumberFormatException e)
        {
            assert(false);
        }
    }

    private int getLevelFromString(String s) {
        if (-1!=s.indexOf("--level=0")) return 0;
        if (-1!=s.indexOf("--level=1")) return 1;
        if (-1!=s.indexOf("--level=2")) return 2;
        if (-1!=s.indexOf("--level=3")) return 3;
        return 0;
    }

    private void fixAttachExeFilename(int size) {
      Vector history = SelectProgram.loadHistory();
      int changed=0;
      for (int i=0; i<history.size(); i++) {
          String s_=(String)history.get(i);
          String s=s_;

          if (size<=11) {
            s=s.replaceAll("six_gtp_engine(-LARGE|-\\d+)?","six_gtp_engine-11");
          }
          else if (size<=15) {
            s=s.replaceAll("six_gtp_engine(-LARGE|-\\d+)?","six_gtp_engine-15");
          }
          else if (size<=19) {
            s=s.replaceAll("six_gtp_engine(-LARGE|-\\d+)?","six_gtp_engine-19");
          }
          if (!s.equals(s_)) { history.set(i, s); changed++; }
      }
      if (changed>0) {
        SelectProgram.saveHistory(history);
      }
    }

    private void cbLevel(int level)
    {
        cbDetachProgram(0);
        cbDetachProgram(1);
        Vector history = SelectProgram.loadHistory();
        //history.add(0, program);
        System.out.println("hsz="+history.size());
        if (history.size()>0) {
            String program=(String)history.get(0);
            System.out.println("hsz="+program);
            int allow_swap=1;
            int allow_resign=1;
            if (-1!=program.indexOf("--allow-swap=0")) allow_swap=0;
            if (-1!=program.indexOf("--allow-swap=1")) allow_swap=1;
            if (-1!=program.indexOf("--allow-resign=0")) allow_resign=0;
            if (-1!=program.indexOf("--allow-resign=1")) allow_resign=1;
            program=program.replaceAll("--allow-swap=0","");
            program=program.replaceAll("--allow-swap=1","");
            program=program.replaceAll("--allow-resign=0","");
            program=program.replaceAll("--allow-resign=1","");
            program=program.replaceAll("--level=0","");
            program=program.replaceAll("--level=1","");
            program=program.replaceAll("--level=2","");
            program=program.replaceAll("--level=3","");
            program=program.trim();
            program+=" --level="+level;
            program+=" --allow-swap="+allow_swap;
            program+=" --allow-resign="+allow_resign;
            System.out.println("hsz="+program);
            history.set(0, program);
            SelectProgram.saveHistory(history);
        }
        cbAttachProgram(0);
    }

    private void cbNextVariation()
    {
        Node node = m_currentNode.getNextVariation();
        if (node != null)
            gotoNode(node);
    }


    private void cbOpen()
    {
        if (m_needsSave && ! checkSaveGame())
            return;
        File file = SimpleDialogs.showOpenSgf(this);
        if (file == null)
            return;
        loadFile(file, -1);
    }

    private void cbOpenRecent()
    {
        if (m_needsSave && ! checkSaveGame())
            return;
        File file = m_menuBar.getSelectedRecent();
        loadFile(file, -1);
    }

    private void cbSwap() {
        humanMoved(new Move(Move.MOVE_SWAP, m_board.getToMove()));
    }


    private void cbPlay()
    {
        if (m_commandThread == null && m_commandThread2 == null) return;
        if (m_board.isGameOver()) return;
        if (m_board.getToMove() == go.Color.BLACK)
            computerBlack();
        else
            computerWhite();
        generateMove();
        m_timeControl.startMove(m_board.getToMove());
    }

    private void cbPreviousVariation()
    {
        Node node = m_currentNode.getPreviousVariation();
        if (node != null)
            gotoNode(node);
    }

    private void cbPrint()
    {
        PrinterJob job = PrinterJob.getPrinterJob();
        job.setPrintable(m_guiBoard);
        if (job.printDialog())
        {
            try
            {
                job.print();
            }
            catch (Exception e)
            {
                showError("Printing failed", e);
            }
            showInfo("Printing done");
        }
    }

    private void cbSaveAs()
    {
        saveDialog();
    }

    private void cbSavePosition()
    {
        File file = SimpleDialogs.showSaveSgf(this);
        if (file == null)
            return;
        try
        {
            if (file.exists())
                if (! showQuestion("Overwrite " + file + "?"))
                    return;
            savePosition(file);
            if (m_currentNode.getFather() == null
                && m_currentNode.getChild() == null)
                m_needsSave = false;
            m_loadedFile = file;
            setTitle();
        }
        catch (FileNotFoundException e)
        {
            showError("Could not save position.", e);
        }
    }

    private void cbSetup()
    {
        if (! m_setupMode)
        {
            if (m_needsSave && ! checkSaveGame())
                return;
            m_menuBar.setSetupMode();
            if (m_gameTreeViewer != null)
            {
                // Create a dummy game tree, so that GameTreeDialog shows
                // a setup node
                m_gameTree = new GameTree(m_boardSize, 0, null, null);
                m_currentNode = m_gameTree.getRoot();
                m_currentNode.addBlack(m_board.getPoint(0, 0));
                m_timeControl.reset();
                updateGameInfo(true);
            }
            resetBoard();
            m_setupMode = true;
            m_toolBar.enableAll(false, null);
            showStatus("Setup Black");
            m_setupColor = go.Color.BLACK;
        }
        else
            setupDone();
    }

    private void cbSetupBlack()
    {
        showStatus("Setup Black");
        m_setupColor = go.Color.BLACK;
        m_board.setToMove(m_setupColor);
        updateGameInfo(false);
    }

    private void cbSetupWhite()
    {
        showStatus("Setup White");
        m_setupColor = go.Color.WHITE;
        m_board.setToMove(m_setupColor);
        updateGameInfo(false);
    }

    private void cbShowAbout()
    {
        AboutDialog.show(this, m_name, m_version);
    }

    /*
    private void cbShowCursor()
    {
        boolean showCursor = m_menuBar.getShowCursor();
        m_guiBoard.setShowCursor(showCursor);
        m_prefs.setBool("show-cursor", showCursor);
        m_guiBoard.repaint();
    }
    */

    private void cbShowLastMove()
    {
        m_showLastMove = m_menuBar.getShowLastMove();
        m_prefs.setBool("show-last-move", m_showLastMove);
        updateGameInfo(false);
    }

    private void cbTruncate()
    {
        if (m_currentNode.getFather() == null)
        {
            showError("Cannot truncate root node.");
            return;
        }
        if (! showQuestion("Truncate current node and all its children?"))
            return;
        Node oldCurrentNode = m_currentNode;
        backward(1);
        m_currentNode.removeChild(oldCurrentNode);
        m_needsSave = true;
        boardChangedBegin(false, true);
    }

    private void checkComputerMove()
    {
        //go.Color toMove = m_board.getToMove();
        final int idx=getCompIdxToMove();
        CommandThread m_commandThread_;
        if (idx==0) m_commandThread_=m_commandThread;
               else m_commandThread_=m_commandThread2;
        m_timeControl.startMove(m_board.getToMove());
        if (m_commandThread_ == null || ! isCurrentNodeExecuted())
            return;
        if (m_computerBlack && m_computerWhite) {
            if (m_board.isGameOver()) { //m_board.bothPassed() || m_resigned
                if (m_auto) {
                    cbSaveAutoFilename(false);
                    newGame(m_boardSize);
                    checkComputerMove();
                    return;
                }
                else {
                    //showInfo("Game finished"); //PHIL
                    computerNone();
                }
                return;
            }
            else
                generateMove();
        }
        else
        {
            if (computerToMove() && !m_board.isGameOver())
                generateMove();
        }
        m_timeControl.startMove(m_board.getToMove());
    }

    private boolean checkCurrentNodeExecuted()
    {
        if (m_commandThread == null)
            return true;
        if (! isCurrentNodeExecuted())
        {
            showError("Could not execute current game node");
            return false;
        }
        return true;
    }

    private boolean checkSaveGame()
    {
        int result = 1; // PHIL
//            JOptionPane.showConfirmDialog(this, "Save game?", "Question",
//                                          JOptionPane.YES_NO_CANCEL_OPTION);
        switch (result)
        {
        case 0:
            return saveDialog();
        case 1:
            m_needsSave = false;
            return true;
        case -1:
        case 2:
            return false;
        default:
            assert(false);
            return true;
        }
    }

    private void clearStatus()
    {
        showStatus(" ");
    }

    private void close()
    {
        if (m_commandInProgress!=0)
            if (! showQuestion("Kill program?"))
                return;
        if (m_setupMode)
            setupDone();
        if (m_needsSave && ! checkSaveGame())
            return;
        saveSession();
        if (m_commandThread != null)
        {
            m_analyzeCommand = null;
            detachProgram(0);
        }
        if (m_commandThread2 != null)
        {
            detachProgram(1);
        }
        dispose();
        m_prefs.save();
        System.exit(0);
    }

    private void computerBlack()
    {
        m_computerBlack = true;
        m_computerWhite = false;
        m_menuBar.setComputerBlack();
    }

    private void computerBoth()
    {
        m_computerBlack = true;
        m_computerWhite = true;
        m_menuBar.setComputerBoth();
    }

    private void computerMoved(int idx)
    {
        CommandThread m_commandThread_;
        if (idx==0) m_commandThread_=m_commandThread;
               else m_commandThread_=m_commandThread2;
        assert(m_commandThread_ != null);
        endLengthyCommand(idx);
        if (m_beepAfterMove)
            java.awt.Toolkit.getDefaultToolkit().beep();
        try
        {
            Gtp.Error e = m_commandThread_.getException();
            if (e != null)
                throw e;
            m_timeControl.stopMove();
            String response = m_commandThread_.getResponse();
            go.Color toMove = m_board.getToMove();
            addPlayerComputerToGameInfo(toMove, idx);

            //go.Point point = Gtp.parsePoint(response, m_boardSize);
            //Move move = new Move(point, toMove);
            Move move = Gtp.parseMove(toMove, response, m_boardSize);
            if (m_board.play(move)) {
              try { //# play on other gtp!!
                if (idx==1 && m_commandThread != null)  m_commandThread.sendCommandPlay(move);
                if (idx==0 && m_commandThread2 != null) m_commandThread2.sendCommandPlay(move);
              }
              catch (Exception e2) {
                //#fixme(2gtp): play might fail
              }

              boolean is_resign=move.getSpecialMove()==Move.MOVE_RESIGN;
              m_needsSave = true;

              Node node = new Node(move);
              m_currentNode.append(node);
              m_currentNode = node;
              m_currentNodeExecuted = 1;
              if (!(m_computerBlack && m_computerWhite)) {
                int spc=move.getSpecialMove();
                if (spc==Move.MOVE_PASS) showInfo(getComputerName(idx) + " passed");
                if (spc==Move.MOVE_SWAP) showInfo(getComputerName(idx) + " swapped");
              }
              //fileModified();
              setTitle();
              //m_resigned = is_resign;
              if (is_resign) {
                if (! (m_computerBlack && m_computerWhite))
                  showInfo(getComputerName(idx) + " resigns");
                setResult((toMove == go.Color.BLACK ? "W" : "B") + "+Resign");
              }

              boardChangedBegin(true, true);
            }
        }
        catch (Gtp.Error e)
        {
            String msg = e.getMessage();
            if ((msg.toLowerCase().indexOf("interrupted")==-1) &&
                (msg.toLowerCase().indexOf("no move found yet")==-1)) {
              showGtpError(e);
            }
            clearStatus();
        }
    }

    private void computerNone()
    {
        m_computerBlack = false;
        m_computerWhite = false;
        m_menuBar.setComputerNone();
    }

    private boolean computerToMove()
    {
        if (m_board.getToMove() == go.Color.BLACK)
            return m_computerBlack;
        else
            return m_computerWhite;
    }

    private void computerWhite()
    {
        m_computerBlack = false;
        m_computerWhite = true;
        m_menuBar.setComputerWhite();
    }

    public static final int WHITE=1; //only for 2nd gtp col
    public static final int BLACK=2;
    public int m_comp2_col=2; //2=second player is default!

    private void computer2Col(int c)
    {
        m_comp2_col=c;
        m_menuBar.setComputer2Col(c);
    }

    private JComponent createStatusBar()
    {
        JPanel outerPanel = new JPanel(new BorderLayout());
        JPanel panel = new JPanel(new GridLayout(1, 0));
        outerPanel.add(panel, BorderLayout.CENTER);
        // Workaround for Java 1.4.1 on Mac OS X: add some empty space
        // so that status bar does not overlap the window resize widget
        if (Platform.isMac())
        {
            Dimension dimension = new Dimension(20, 1);
            Box.Filler filler =
                new Box.Filler(dimension, dimension, dimension);
            outerPanel.add(filler, BorderLayout.EAST);
        }
        JLabel label = new JLabel();
        label.setBorder(BorderFactory.createLoweredBevelBorder());
        label.setHorizontalAlignment(SwingConstants.LEFT);
        panel.add(label);
        m_statusLabel = label;
        clearStatus();
        return outerPanel;
    }

    private void detachProgram(int idx)
    {
        CommandThread m_commandThread_;
        GtpShell m_gtpShell_;
        if (idx==0) { m_commandThread_=m_commandThread; m_gtpShell_=m_gtpShell; }
               else { m_commandThread_=m_commandThread2; m_gtpShell_=m_gtpShell2; }
        if (m_commandInProgress!=0)
        {
            m_commandThread_.destroyGtp();
            m_commandThread_.close();
        }
        else
        {
            if (m_commandThread_ != null && ! m_commandThread_.isProgramDead())
            {
                // Some programs do not handle closing the GTP stream
                // correctly, so we send a quit before
                try
                {
                    m_commandThread_.sendCommand("quit");
                }
                catch (Gtp.Error e)
                {
                }
                m_commandThread_.close();
            }
        }
        if (m_analyzeCommand != null)
            clearAnalyzeCommand();
        if (idx==0) m_commandThread = null;
               else m_commandThread2 = null;
        if (idx==0) {
          m_name = "";
          m_version = "";
        } else {
          m_name2 = "";
          m_version2 = "";
        }
        m_toolBar.setComputerEnabled(m_commandThread!=null, m_commandThread2!=null); //false
        m_menuBar.setComputerEnabled(m_commandThread!=null, m_commandThread2!=null); //false
        m_gtpShell_.dispose();
        if (idx==0) m_gtpShell = null;
               else m_gtpShell2 = null;
        if (m_analyzeDialog != null)
        {
            m_analyzeDialog.saveRecent();
            m_analyzeDialog.dispose();
            m_analyzeDialog = null;
        }
        resetBoard();
        clearStatus();
        setTitle();
        // If this node was only partially executed due to a error of the Go
        // program, we undo and execute it again
        if (! isCurrentNodeExecuted())
        {
            try
            {
                undoCurrentNode();
                executeCurrentNode();
            }
            catch (Gtp.Error e)
            {
                assert(false);
            }
        }
    }

    private void executeCurrentNode() throws Gtp.Error
    {
        m_currentNodeExecuted = 0;
        Vector moves = m_currentNode.getAllAsMoves();
        for (int i = 0; i < moves.size(); ++i)
        {
            Move move = (Move)moves.get(i);
            if (m_board.play(move)) { //#BUGFIX ?!
                if (m_commandThread != null)  m_commandThread.sendCommandPlay(move);
                if (m_commandThread2 != null) m_commandThread2.sendCommandPlay(move);
                ++m_currentNodeExecuted;
            } else {

                break;
            }
        }
        go.Color toMove = m_currentNode.getToMove();
        if (toMove != go.Color.EMPTY)
            m_board.setToMove(toMove);
        m_guiBoard.updateFromGoBoard();
    }

    private boolean executeRoot()
    {
        m_currentNode = m_gameTree.getRoot();
        while (m_board.undo()) ; //#BUGFIX ?!
        m_currentNodeExecuted = 0;
        m_isRootExecuted = true;
        if (m_commandThread != null)
        {
            try
            {
                m_commandThread.sendCommandBoardsize(m_boardSize);
                m_commandThread.sendCommandClearBoard(m_boardSize);
            }
            catch (Gtp.Error error)
            {
                showGtpError(error);
                m_isRootExecuted = false;
                return false;
            }
        }
        if (m_commandThread2 != null)
        {
            try
            {
                m_commandThread2.sendCommandBoardsize(m_boardSize);
                m_commandThread2.sendCommandClearBoard(m_boardSize);
            }
            catch (Gtp.Error error)
            {
                showGtpError(error);
                m_isRootExecuted = false;
                return false;
            }
        }
        GameInformation gameInformation = m_gameTree.getGameInformation();
        setKomi(gameInformation.m_komi);
        setRules();
        setTimeSettings();
        try
        {
            executeCurrentNode();
        }
        catch (Gtp.Error error)
        {
            showGtpError(error);
            return false;
        }
        return true;
    }

    private void fileModified()
    {
        if (m_loadedFile != null)
        {
            m_loadedFile = null;
            setTitle();
        }
    }

    //NEW: try to go fwd (if subnode exists)
    private boolean cbForward2(Move m)
    {
        if (! checkCurrentNodeExecuted())
            return false;
        setFastUpdate(true);
        try
        {
            for (int i = 0; i < m_currentNode.getNumberChildren(); ++i) {
                Node n2 = m_currentNode.getChild(i);
                if (n2.getMove().equals(m)) {
                  m_currentNode=n2;
                  executeCurrentNode();
                  boardChangedBegin(false, false);
                  return true;
                }
            }
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        finally {
          setFastUpdate(false);
        }
        return false;
    }

    private void forward(int n)
    {
        if (! checkCurrentNodeExecuted())
            return;
        setFastUpdate(true);
        try
        {
            for (int i = 0; i < n && m_currentNode.getNumberChildren() > 0;
                 ++i)
            {
                m_currentNode = m_currentNode.getChild();
                executeCurrentNode();
            }
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        setFastUpdate(false);
    }

    //0 or 1 ; -1 = no computer attached
    int getCompIdxToMove() {
      go.Color toMove = m_board.getToMove();
      int idx;
           if (m_commandThread!=null && m_commandThread2==null) idx=0;
      else if (m_commandThread==null && m_commandThread2!=null) idx=1;
      else if (m_commandThread==null && m_commandThread2==null) return -1;
      //else idx=(toMove==go.Color.WHITE) ? 0 : 1;
      else idx=(toMove==go.Color.WHITE ?
          (m_comp2_col==WHITE ? 1 : 0) :
          (m_comp2_col==BLACK ? 1 : 0)); //both attached

      return idx;
    }

    private void generateMove()
    {
        if (m_commandInProgress!=0) return; //#BUGFIX ?!
        go.Color toMove = m_board.getToMove();
        String command;
        final int idx=getCompIdxToMove();
        showStatus((idx==0?"GTP: ":"2nd GTP: ")+getComputerName(idx) + " is thinking ...");
        if (idx==-1) return;
        if (idx==0) command = m_commandThread.getCommandGenmove(toMove);
               else command = m_commandThread2.getCommandGenmove(toMove);
        Runnable callback = new Runnable()
            {
                public void run() { computerMoved(idx); }
            };
        runLengthyCommand(idx, command, callback);
    }

    private String getComputerName(int i)
    {
      if (i==0 && m_name != null && ! m_name.equals("")) return StringUtils.capitalize(m_name);
      if (i==1 && m_name2 != null && ! m_name2.equals("")) return StringUtils.capitalize(m_name2);
      return "Computer";
    }

    private int getRules()
    {
        int result = go.GameBoard.RULES_UNKNOWN;
        String rules = m_gameTree.getGameInformation().m_rules;
        if (rules != null)
        {
            rules = rules.trim().toLowerCase();
            if (rules.equals("japanese"))
                result = go.GameBoard.RULES_JAPANESE;
            else if (rules.equals("chinese"))
                result = go.GameBoard.RULES_CHINESE;
        }
        return result;
    }

    //as children
    private boolean addMovesInternal2Children(ArrayList<Move> vv) {
        if (vv==null) return true;
        System.out.println("Adding2 "+vv.size());
        for (int i=0; i<vv.size(); i++) {
            Move move=vv.get(i);
            System.out.println("Adding2 nr. "+(i+1)+" "+move.toString());
            //go.Point p = move.getPoint();
            //go.Color c = move.getColor();

            if (m_board.play(move)) { // first try it
                m_board.undo();

                Node node = new Node(move);
                m_currentNode.append(node);
            }
        }
        boardChangedBegin(false, true);

        return true;

    }

    private boolean addMovesInternal(ArrayList<Move> vv) {
        if (vv==null) return true;
        try {
            for (int i=0;i<vv.size();i++) {
                Move move=vv.get(i);
                System.out.println("Adding nr. "+(i+1)+" "+move.toString());
                go.Point p = move.getPoint();
                go.Color c = move.getColor();

                if (m_board.play(move)) { // first try it
                    m_board.undo();
                } else {
                    return false;
                }
                try
                {
                    setFastUpdate(true);
                    m_needsSave = true;
                    play(move);
                }
                finally
                {
                    setFastUpdate(false);
                }
                if (p != null)
                {

                    if (m_showLastMove)
                        m_guiBoard.markLastMove(move.getPoint());

                    paintImmediately(p);
                }
                m_timeControl.stopMove();

                //boolean last=(i==vv.size()-1);
                boardChangedBegin(false, true);

            }

            return true;
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
        return false;
    }

    private void humanMoved(Move move)
    {
        //#feature deactivated
        /*if (move.isNormalMove() && m_board.pegs[move.getX()][move.getY()]!=0) { // if peg exists => navigate backwards
          cbBackward2(move); return;
        }*/
        if (m_board.isGameOver()) return;
        if (cbForward2(move)) return; // if subnode exists => do not add new node!!
        //if (move!=null && move.getColor()==go.Color.EMPTY) return;
        try {
            go.Point point = move.getPoint();
            int spc=move.getSpecialMove();

            // regular Move illegal? (nonempty point
            if (point!=null) {
              if (m_board.getColor(point) != go.Color.EMPTY) return;
            }
            // swap illegal?
            if (spc==Move.MOVE_SWAP && m_board.getMoveNumber()!=1) return;

            try
            {
                setFastUpdate(true);
                m_needsSave = true;
                play(move);
            }
            finally
            {
                setFastUpdate(false);
            }
            if (point != null)
            {
                m_guiBoard.updateFromGoBoard(point);

                if (m_showLastMove)
                    m_guiBoard.markLastMove(move.getPoint());

                paintImmediately(point);
            }
            m_timeControl.stopMove();
            go.Color color = move.getColor();
            if (m_board.getMoveNumber() > 0
                && m_timeControl.lostOnTime(color)
                && ! m_lostOnTimeShown)
            {
                showInfo(color.toString() + " lost on time.");
                m_lostOnTimeShown = true;
            }
            //m_resigned = false;
            boardChangedBegin(true, true);
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
    }

    private void initGame(int size)
    {
        if (size != m_boardSize)
        {
            m_boardSize = size;
            m_guiBoard.initSize(size);
            //m_squareLayout.setPreferMultipleOf(2 + 2 * size);
            //pack();
            /*
            restoreMainWindow();
            if (m_gtpShell != null)
                restoreSize(m_gtpShell, "window-gtpshell", m_boardSize);
            if (m_analyzeDialog != null)
                restoreSize(m_analyzeDialog, "window-analyze", m_boardSize);
            if (m_gameTreeViewer != null)
                restoreSize(m_gameTreeViewer, "window-gametree", m_boardSize);
            */
        }
        Vector handicap = m_board.getHandicapStones(m_handicap);
        if (handicap == null)
            showWarning("Handicap stone locations are not\n" +
                        "defined for this board size.");
        m_gameTree = new GameTree(size, m_prefs.getFloat("komi"), handicap,
                                  m_prefs.getString("rules"));
        m_board.newGame();
        m_currentNode = m_gameTree.getRoot();
        m_currentNodeExecuted = 0;
        m_guiBoard.updateFromGoBoard();
        resetBoard();
        m_timeControl.reset();
        m_timeControl.startMove(go.Color.BLACK);
        m_lostOnTimeShown = false;
        m_needsSave = false;
        //m_resigned = false;
    }

    private void initialize()
    {
        if (! m_file.equals(""))
            newGameFile(m_boardSize, m_move);
        else
            newGame(m_boardSize);
        m_toolBar.enableAll(true, m_currentNode);
        if (m_program != null) attachProgram(0, m_program);
        if (m_program2 != null) {
          if (OPTION_2nd_gtp)
            attachProgram(1, m_program2);
          else
            m_program2=null;
        }
        setTitle();
        if (m_commandThread == null
            || (! m_computerBlack && ! m_computerWhite))
            computerNone();
        else if (m_computerBlack && m_computerWhite)
            computerBoth();
        else  if (m_computerBlack)
            computerBlack();
        else
            computerWhite();
        computer2Col(WHITE);
        if (m_prefs.getBool("show-gametree"))
            cbShowGameTree();
        if (! m_initAnalyze.equals(""))
        {
            AnalyzeCommand analyzeCommand =
                AnalyzeCommand.get(this, m_initAnalyze);
            if (analyzeCommand == null)
                showError("Unknown analyze command \"" + m_initAnalyze + "\"");
            else
                initAnalyzeCommand(analyzeCommand, true);
        }
        updateGameInfo(true);
        setVisible(true);
        m_guiBoard.setFocus();
        setTitleFromProgram();
        checkComputerMove();
    }

    private void initScore(go.Point[] isDeadStone)
    {
        resetBoard();
        m_guiBoard.scoreBegin(isDeadStone);
        m_guiBoard.repaint();
        m_scoreMode = true;
        if (m_scoreDialog == null)
            m_scoreDialog = new ScoreDialog(this, this);
        float komi = m_gameTree.getGameInformation().m_komi;
        m_scoreDialog.showScore(m_board.scoreGet(komi, getRules()));
        m_scoreDialog.setVisible(true);
        m_menuBar.setScoreMode();
        showStatus("Please mark dead groups.");
    }

    private boolean isCurrentNodeExecuted()
    {
        int numberAddStonesAndMoves = m_currentNode.getAllAsMoves().size();
        if (! m_isRootExecuted
            || m_currentNodeExecuted != numberAddStonesAndMoves)
            return false;
        return true;
    }

    String currentDir="";
    private void loadFile(File file, int move)
    {
        java.io.Reader fileReader=null;
        try
        {
            m_menuBar.addRecent(file);
            m_menuBar.saveRecent();
            fileReader = new FileReader(file);
            sgf.Reader reader = new sgf.Reader(fileReader, file.toString());
            currentDir=file.getParent().toString();
            GameInformation gameInformation =
                reader.getGameTree().getGameInformation();
            initGame(gameInformation.m_boardSize);
            m_gameTree = reader.getGameTree();
            executeRoot();
            if (move > 0)
                forward(move);
            m_loadedFile = file;
            setTitle();
            SimpleDialogs.setLastFile(file);
            computerNone();
            boardChangedBegin(false, true);
        }
        catch (FileNotFoundException e)
        {
            showError("File not found:\n" + file);
        }
        catch (sgf.Reader.Error e)
        {
            showError("Could not read file.", e);
        }
        finally { //#BUGFIX: do not block file
          try {
            fileReader.close();
          } catch (IOException e) {
            e.printStackTrace();
          }
        }
    }

    private void newGame(int size)
    {
        initGame(size);
        executeRoot();
        updateGameInfo(true);
        m_guiBoard.updateFromGoBoard();
        m_toolBar.updateGameButtons(m_currentNode);
        m_menuBar.updateGameMenuItems(m_gameTree, m_currentNode);
        m_menuBar.selectBoardSizeItem(m_board.getSize());
        setTitle();
        setTitleFromProgram();
    }

    private void newGameFile(int size, int move)
    {
        initGame(size);
        loadFile(new File(m_file), move);
        m_timeControl.halt();
        updateGameInfo(true);
        m_guiBoard.updateFromGoBoard();
        m_toolBar.updateGameButtons(m_currentNode);
        m_menuBar.updateGameMenuItems(m_gameTree, m_currentNode);
        m_menuBar.selectBoardSizeItem(m_board.getSize());
    }

    /** Paint point and some neighbors (because of the shadows)
        immediately to pretend better responsiveness
    */
    private void paintImmediately(go.Point point)
    {
        m_guiBoard.paintImmediately(point);
    }

    private void play(Move move) throws Gtp.Error
    {
        if (! checkCurrentNodeExecuted())
            return;
        Node node = new Node(move);
        m_currentNode.append(node);
        m_currentNode = node;
        try
        {
            executeCurrentNode();
        }
        catch (Gtp.Error error)
        {
            m_currentNode = node.getFather();
            m_currentNode.removeChild(node);
            m_currentNodeExecuted = m_currentNode.getAllAsMoves().size();
            throw error;
        }
    }

    private void resetBoard()
    {
        clearStatus();
        m_guiBoard.resetBoard();
        m_guiBoard.updateFromGoBoard();
        m_guiBoard.repaint();
    }

    private void restoreMainWindow()
    {
        restoreSize(this, "window-gogui", m_boardSize);
        String name = "splitpane-position-" + m_boardSize;
        if (m_prefs.contains(name))
        {
            int dividerLocation = m_prefs.getInt(name);
            m_splitPane.setDividerLocation(dividerLocation);
        }
    }

    private void restoreSize(Window window, String name, int size)
    {
        name = name; // + "-" + size;
        if (! m_prefs.contains(name))
            return;
        String[] tokens = StringUtils.tokenize(m_prefs.getString(name));
        if (tokens.length < 4)
            return;
        try
        {
            Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
            int x = Math.max(0, Integer.parseInt(tokens[0]));
            if (x > screenSize.width)
                x = 0;
            int y = Math.max(0, Integer.parseInt(tokens[1]));
            if (y > screenSize.height)
                y = 0;
            int width = Integer.parseInt(tokens[2]);
            width = Math.min(width, screenSize.width);
            int height = Integer.parseInt(tokens[3]);
            height = Math.min(height, screenSize.height);
            if (window instanceof GtpShell)
                ((GtpShell)window).setFinalSize(x, y, width, height);
            else
            {
                window.setBounds(x, y, width, height);
                window.validate();
            }
        }
        catch (NumberFormatException e)
        {
        }
    }

    private void runLengthyCommand(int idx, String cmd, Runnable callback)
    {
        if (idx==0) {
          assert(m_commandThread != null);
          beginLengthyCommand(idx);
          m_commandThread.sendCommand(cmd, callback);
        }
        else {
          assert(m_commandThread2 != null);
          beginLengthyCommand(idx);
          m_commandThread2.sendCommand(cmd, callback);
        }
    }

    private void save(File file, boolean addRecent) throws FileNotFoundException
    {
        OutputStream out = new FileOutputStream(file);
        if (FileUtils.hasExtension(file, "tex"))
        {
            //String title = FileUtils.removeExtension(new File(file.getName()), //#
            //                                                  "tex");
            //new latex.Writer(title, out, m_gameTree, false);
        }
        else
        {
            new sgf.Writer(out, m_gameTree, file, "HexGui",  Version.getVersion());
            if (addRecent) {
              m_menuBar.addRecent(file);
              m_menuBar.saveRecent();
            }
        }
    }

    // convert int to string (with leading zeros)
    public static String intToString_leadZeros(int val, int width) {
        String s=""+val;
        while (s.length()<width) s="0"+s;
        return s;
    }

    public static String currentTimeToString()
    {
        Calendar c = Calendar.getInstance();
        return
            intToString_leadZeros(c.get(Calendar.YEAR),4)+"-"+
            intToString_leadZeros(c.get(Calendar.MONTH)+1,2)+"-"+         // 0..11 !!
            intToString_leadZeros(c.get(Calendar.DAY_OF_MONTH),2)+"--"+   // 1..31 ok
            intToString_leadZeros(c.get(Calendar.HOUR_OF_DAY),2)+"-"+
            intToString_leadZeros(c.get(Calendar.MINUTE),2)+"-"+
            intToString_leadZeros(c.get(Calendar.SECOND),2);
    }

    private void cbSaveAutoFilename(boolean addRecent)
    {
        try
        {
            File curdir=new File(currentDir);
            String fname="HGui-save_"+currentTimeToString()+"."+Version.SgfExt();
            File file=new File(fname); //current working directory
            if (curdir.exists()) file=new File(currentDir, fname); //in dir of last opened file
            save(file, addRecent);
            m_loadedFile = file;
            setTitle();
            m_needsSave = false;
            System.err.println("Saved:"+file);
        }
        catch (FileNotFoundException e)
        {
            showError("Could not save game.", e);
        }
    }

    private void cbSave()
    {
        try
        {
            if ((m_loadedFile!=null)&&(!m_loadedFile.equals(""))) {
                save(m_loadedFile, true);
                setTitle();
                m_needsSave = false;
            } else {
                cbSaveAs();
            }
        }
        catch (FileNotFoundException e)
        {
            showError("Could not save game.", e);
        }
    }

    private boolean saveDialog()
    {
        try
        {
            File file;
            while (true)
            {
                file = SimpleDialogs.showSaveSgf(this);
                if (file == null)
                    return false;
                if (file.exists())
                    if (! showQuestion("Overwrite " + file + "?"))
                        continue;
                break;
            }
            save(file, true);
            m_loadedFile = file;
            setTitle();
            m_needsSave = false;
            return true;
        }
        catch (FileNotFoundException e)
        {
            showError("Could not save game.", e);
            return false;
        }
    }

    private void savePosition(File file) throws FileNotFoundException
    {
        OutputStream out = new FileOutputStream(file);
        if (FileUtils.hasExtension(file, "tex"))
        {
            String title = FileUtils.removeExtension(new File(file.getName()),
                                                     "tex");
        }
        else
        {
            new sgf.Writer(out, m_board, file, "HexGui", Version.getVersion());
            m_menuBar.addRecent(file);
            m_menuBar.saveRecent();
        }
    }

    private void saveSession()
    {
        if (m_gtpShell != null)
            m_gtpShell.saveHistory();
        m_menuBar.saveRecent();
        if (m_analyzeDialog != null)
            m_analyzeDialog.saveRecent();
        saveSize(this, "window-gogui");
        if (m_help != null)
            saveSize(m_help, "window-help");
        if (m_gameTreeViewer != null)
            saveSizeAndVisible(m_gameTreeViewer, "gametree");
        if (m_commandThread != null)
        {
            saveSizeAndVisible(m_gtpShell, "gtpshell");
            saveSizeAndVisible(m_gtpShell2, "gtpshell2");
            saveSizeAndVisible(m_analyzeDialog, "analyze");
        }
        if (GuiUtils.isNormalSizeMode(this))
        {
            String name = "splitpane-position-" + m_boardSize;
            m_prefs.setInt(name, m_splitPane.getDividerLocation());
        }
    }

    private void saveSize(JFrame window, String name)
    {
        if (! GuiUtils.isNormalSizeMode(window))
            return;
        name = name; // + "-" + m_boardSize;
        java.awt.Point location = window.getLocation();
        Dimension size = window.getSize();
        String value = Integer.toString(location.x) + " " + location.y
            + " " + size.width + " " + size.height;
        m_prefs.setString(name, value);
    }

    private void saveSizeAndVisible(JFrame window, String name)
    {
        if (window != null)
            saveSize(window, "window-" + name);
        boolean isVisible = (window != null && window.isVisible());
        m_prefs.setBool("show-" + name, isVisible);
    }

    private void sendGtpString(String commands)
    {
        commands = commands.replaceAll("\\\\n", "\n");
        m_gtpShell.sendGtp(new StringReader(commands));
    }

    private void sendInterrupt(int idx)
    {
        if (m_commandInProgress==0) return;
        showStatus("Interrupting ...");
        try
        {
            if (idx==0) m_commandThread.sendInterrupt();
                   else m_commandThread2.sendInterrupt();
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
    }

    private void setBoardCursor(int type)
    {
        setCursor(m_guiBoard /*m_boardPanel*/, type);
        setCursor(m_infoPanel, type);
    }

    private void setBoardCursorDefault()
    {
        setCursorDefault(m_guiBoard /*m_boardPanel*/);
        setCursorDefault(m_infoPanel);
    }

    private void setCursor(Component component, int type)
    {
        Cursor cursor = Cursor.getPredefinedCursor(type);
        component.setCursor(cursor);
    }

    private void setCursorDefault(Component component)
    {
        component.setCursor(Cursor.getDefaultCursor());
    }

    private void setFastUpdate(boolean fastUpdate)
    {
        if (m_commandThread != null)
        {
            assert(m_commandThread.getFastUpdate() != fastUpdate);
            m_commandThread.setFastUpdate(fastUpdate);
            m_gtpShell.setFastUpdate(fastUpdate);
        }
    }

    private void setKomi(float komi)
    {
        if (m_commandThread == null)
            return;
        try
        {
            if (m_commandThread.isCommandSupported("komi"))
                m_commandThread.sendCommand("komi " + komi);
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
    }

    private static void setPrefsDefaults(Preferences prefs)
    {
        prefs.setBoolDefault("beep-after-move", false);
        prefs.setIntDefault("boardsize", Version.defaultBoardSize);
        prefs.setFloatDefault("komi", 0);
        prefs.setStringDefault("rules", "Chinese");
        prefs.setBoolDefault("show-analyze", false);
        prefs.setBoolDefault("show-gtpshell", true);
        prefs.setBoolDefault("show-gtpshell2", true);
        prefs.setBoolDefault("show-gametree", true);
        prefs.setBoolDefault("show-cursor", false);
        prefs.setBoolDefault("show-last-move", true);
    }

    private void setResult(String result)
    {
        String oldResult = m_gameTree.getGameInformation().m_result;
        if (! (oldResult == null || oldResult.equals("")
               || oldResult.equals(result))
            && ! showQuestion("Overwrite old result " + oldResult + "\n" +
                              "with new score " + result + "?"))
            return;
        m_gameTree.getGameInformation().m_result = result;
    }

    private void setRules()
    {
        if (m_commandThread == null)
            return;
        int rules = getRules();
        if (rules == go.GameBoard.RULES_UNKNOWN)
            return;
        if (! m_commandThread.isCommandSupported("scoring_system"))
            return;
        try
        {
            String s =
                (rules == go.GameBoard.RULES_JAPANESE ? "territory" : "area");
            m_commandThread.sendCommand("scoring_system " + s);
        }
        catch (Gtp.Error e)
        {
        }
    }

    private void setTimeSettings()
    {
        if (m_commandThread == null)
            return;
        if (! m_timeControl.isInitialized())
            return;
        if (! m_commandThread.isCommandSupported("time_settings"))
        {
            showError("Command time_settings not supported");
            return;
        }
        long preByoyomi = m_timeControl.getPreByoyomi() * 60;
        long byoyomi = 0;
        long byoyomiMoves = 0;
        if (m_timeControl.getUseByoyomi())
        {
            byoyomi = m_timeControl.getByoyomi() * 60;
            byoyomiMoves = m_timeControl.getByoyomiMoves();
        }
        try
        {
            m_commandThread.sendCommand("time_settings " + preByoyomi + " "
                                        + byoyomi + " " + byoyomiMoves);
        }
        catch (Gtp.Error e)
        {
            showGtpError(e);
        }
    }

    private void setTitle()
    {
        //String appName = "GoGui";
        String appName = Version.AppName()+" "+Version.getVersion()+" (based on GoGui)";
        if (m_commandThread2 != null && ! m_name2.equals("")) appName = getComputerName(1);
        if (m_commandThread != null && ! m_name.equals("")) appName = getComputerName(0);
        if (m_gtpShell != null) m_gtpShell.setAppName("GTP - "+getComputerName(0));
        if (m_gtpShell2 != null) m_gtpShell2.setAppName("2nd GTP - "+getComputerName(1));
        if (m_analyzeDialog != null)
            m_analyzeDialog.setAppName(appName);
        String gameName = null;
        GameInformation gameInformation = m_gameTree.getGameInformation();
        String playerBlack = gameInformation.m_playerBlack;
        String playerWhite = gameInformation.m_playerWhite;
        if (playerBlack != null && ! playerBlack.trim().equals("")
            && playerWhite != null && ! playerWhite.trim().equals(""))
        {
            playerBlack = StringUtils.capitalize(playerBlack);
            playerWhite = StringUtils.capitalize(playerWhite);
            String blackRank = gameInformation.m_blackRank;
            String whiteRank = gameInformation.m_whiteRank;
            if (blackRank != null && ! blackRank.trim().equals(""))
                playerBlack = playerBlack + " [" + blackRank + "]";
            if (whiteRank != null && ! whiteRank.trim().equals(""))
                playerWhite = playerWhite + " [" + whiteRank + "]";
            if (m_loadedFile != null)
                gameName = m_loadedFile.getName() + " - "
                    + playerBlack + "(B) vs " + playerWhite + "(W)";
            else
                gameName = playerBlack + "(B) vs " + playerWhite + "(W)";
        }
        else if (m_loadedFile != null)
            gameName= m_loadedFile.getName();
        if (gameName != null)
            setTitle(gameName + " - " + appName);
        else
            setTitle(appName);
    }

    private void setTitleFromProgram()
    {
        if (m_commandThread == null)
            return;
        if (m_commandThread.isCommandSupported("gogui_title"))
        {
            try
            {
                String title = m_commandThread.sendCommand("gogui_title");
                setTitle(title);
            }
            catch (Gtp.Error e)
            {
            }
        }
    }

    private void setupDone()
    {
        setFastUpdate(true);
        m_setupMode = false;
        m_menuBar.setNormalMode();
        m_toolBar.enableAll(true, m_currentNode);
        int size = m_board.getSize();
        go.Color color[][] = new go.Color[size][size];
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point p = m_board.getPoint(i);
            color[p.getX()][p.getY()] = m_board.getColor(p);
        }
        go.Color toMove = m_board.getToMove();
        m_boardSize = size;
        m_board.newGame();
        m_gameTree = new GameTree(size, m_prefs.getFloat("komi"), null,
                                  m_prefs.getString("rules"));
        m_currentNode = m_gameTree.getRoot();
        for (int i = 0; i < m_board.getNumberPoints(); ++i)
        {
            go.Point point = m_board.getPoint(i);
            int x = point.getX();
            int y = point.getY();
            go.Color c = color[x][y];
            if (c == go.Color.BLACK)
                m_currentNode.addBlack(point);
            else if (c == go.Color.WHITE)
                m_currentNode.addWhite(point);
        }
        m_currentNode.setPlayer(toMove);
        executeRoot();
        fileModified();
        updateGameInfo(true);
        boardChangedBegin(false, false);
        setFastUpdate(false);
    }

    private void showError(String message, Exception e)
    {
        SimpleDialogs.showError(this, message, e);
    }

    private void showError(String message)
    {
        SimpleDialogs.showError(this, message);
    }

    private void showGtpError(Gtp.Error e)
    {
        showGtpError(this, e);
    }

    private void showGtpError(Component frame, Gtp.Error e)
    {
        String message = e.getMessage().trim();
        if (message.length() == 0)
            message = "Command failed";
        else
            message = StringUtils.capitalize(message);
        SimpleDialogs.showError(frame, message);
    }

    private void showInfo(String message)
    {
        SimpleDialogs.showInfo(this, message);
    }

    private boolean showQuestion(String message)
    {
        return SimpleDialogs.showQuestion(this, message);
    }

    private void showStatus(String text)
    {
        m_statusLabel.setText(text);
        m_statusLabel.repaint();
    }

    private void showStatusSelectPointList()
    {
        showStatus("Select points for " + m_analyzeCommand.getLabel()
                   + " (last point with right button or Ctrl key down)");
    }

    private void showStatusSelectTarget()
    {
        showStatus("Select a target for " + m_analyzeCommand.getResultTitle());
    }

    private void showWarning(String message)
    {
        SimpleDialogs.showWarning(this, message);
    }

    private void undoCurrentNode() throws Gtp.Error
    {
        for ( ; m_currentNodeExecuted > 0; --m_currentNodeExecuted)
        {
            if (m_commandThread != null) m_commandThread.sendCommand("undo");
            if (m_commandThread2 != null) m_commandThread2.sendCommand("undo");
            m_board.undo();
        }
        m_guiBoard.updateFromGoBoard();
    }

    private void updateGameInfo(boolean gameTreeChanged)
    {
        boolean end_game = m_gameInfo.update(m_currentNode, m_board);

        if (m_gameTreeViewer != null)
        {
            if (gameTreeChanged)
                m_gameTreeViewer.update(m_gameTree, m_currentNode);
            else
                m_gameTreeViewer.update(m_currentNode);
        }
        m_comment.setNode(m_currentNode);
        if (m_showLastMove && checkCurrentNodeExecuted())
        {
            Move move = m_currentNode.getMove();
            if (move != null)
                m_guiBoard.markLastMove(move.getPoint());
            else
                m_guiBoard.markLastMove(null);
        }
        else
            m_guiBoard.markLastMove(null);
        if (m_analyzeDialog != null)
            m_analyzeDialog.setSelectedColor(m_board.getToMove());

        if (end_game) {
            //m_guiBoard.showBWBoard(m_board);  // PHIL
            //for (int i = 1; i < 1000; i++) {
                if (m_board.getConnectWinner() == go.Color.BLACK) {
                    //m_guiBoard.ColorWhite = new java.awt.Color(0,0,100); //java.awt.Color.green;
                    m_guiBoard.DrawLoose = 1;
                } else {
                    //m_guiBoard.ColorBlack = new java.awt.Color(100,0,0); //java.awt.Color.green;
                    m_guiBoard.DrawLoose = 2;
                }
                //m_guiBoard.repaint();

//                if (m_board.getConnectWinner() == go.Color.BLACK) {
//                    m_guiBoard.ColorBlack = java.awt.Color.red;
//                } else {
//                    m_guiBoard.ColorWhite = java.awt.Color.blue;
//                }
//                m_guiBoard.repaint();

                //}
        } else {
            m_guiBoard.ColorBlack = java.awt.Color.red;
            m_guiBoard.ColorWhite = java.awt.Color.blue;
            m_guiBoard.DrawLoose = 0;
        }
    }
}

//----------------------------------------------------------------------------
