//----------------------------------------------------------------------------
// $Id: MenuBar.java,v 1.67 2004/07/09 17:31:09 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/gui/MenuBar.java,v $
//----------------------------------------------------------------------------

package gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import version.Version;
import game.*;

//----------------------------------------------------------------------------

class MenuBar
    implements ActionListener
{
    public MenuBar(ActionListener listener)
    {
        m_listener = listener;
        m_menuBar = new JMenuBar();
        m_menuFile = createFileMenu();
        m_menuBar.add(m_menuFile);
        m_menuGame = createGameMenu();
        m_menuBar.add(m_menuGame);
        m_menuVariation = createVariationMenu();
        m_menuBar.add(m_menuVariation);
        m_menuSettings = createSettingsMenu();
        m_menuBar.add(m_menuSettings);
        m_menuWindow = createWindowMenu();
        m_menuBar.add(m_menuWindow);

        // NEW:
        m_menu2ndGTP = create2ndGTPMenu();
        if (HGui.OPTION_2nd_gtp) m_menuBar.add(m_menu2ndGTP);

        m_menuHelp = createHelpMenu();
        m_menuBar.add(m_menuHelp);
    }

    public void actionPerformed(ActionEvent event)
    {
        String command = event.getActionCommand();
        if (command.startsWith("open-recent-"))
        {
            try
            {
                String indexString =
                    command.substring("open-recent-".length());
                int index = Integer.parseInt(indexString);
                m_selectedRecent = m_recent[index];
                assert(m_selectedRecent != null);
                m_listener.actionPerformed(new ActionEvent(this, 0,
                                                           "open-recent"));
            }
            catch (NumberFormatException e)
            {
                assert(false);
                return;
            }
        }
        else
            assert(false);
    }

    public void addRecent(File file)
    {
        try
        {
            File canonicalFile = file.getCanonicalFile();
            if (canonicalFile.exists())
                file = canonicalFile;
        }
        catch (IOException e)
        {
        }
        for (int i = 0; i < m_maxRecent; ++i)
        {
            if (m_recent[i] == null)
                break;
            if (m_recent[i].equals(file))
            {
                for (int j = i; j > 0; --j)
                    m_recent[j] = m_recent[j - 1];
                m_recent[0] = file;
                updateRecentMenu();
                return;
            }
        }
        for (int i = m_maxRecent - 1; i > 0; --i)
            m_recent[i] = m_recent[i - 1];
        m_recent[0] = file;
        updateRecentMenu();
    }

    public boolean getBeepAfterMove()
    {
        return m_itemBeepAfterMove.isSelected();
    }

    public JMenuBar getMenuBar()
    {
        return m_menuBar;
    }

    public File getSelectedRecent()
    {
        return m_selectedRecent;
    }

    /*
    public boolean getShowCursor()
    {
        return m_itemShowCursor.isSelected();
    }
    */

    public boolean getShowLastMove()
    {
        return m_itemShowLastMove.isSelected();
    }

    public void saveRecent()
    {
        File file = getRecentFile();
        PrintStream out;
        try
        {
            out = new PrintStream(new FileOutputStream(file));
        }
        catch (FileNotFoundException e)
        {
            System.err.println("Could not save " + file.toString());
            return;
        }
        for (int i = 0; i < m_maxRecent; ++i)
            if (m_recent[i] != null)
                out.println(m_recent[i].toString());
        out.close();
    }

    public void setComputerBlack()
    {
        m_itemComputerBlack.setSelected(true);
    }

    public void setComputerBoth()
    {
        m_itemComputerBoth.setSelected(true);
    }

    public void setComputerNone()
    {
        m_itemComputerNone.setSelected(true);
    }

    public void setComputerWhite()
    {
        m_itemComputerWhite.setSelected(true);
    }

    public void setComputer2Col(int v)
    {
        if (v==HGui.WHITE) m_itemComputerWhite2.setSelected(true);
                      else m_itemComputerBlack2.setSelected(true);
    }

    public void selectLevelItem(int size)
    {
        for (int i = 0; i < m_possibleLevels.length; ++i)
            if (m_possibleLevels[i] == size)
            {
                m_itemLevel[i].setSelected(true);
                return;
            }
    }

    public void selectBoardSizeItem(int size)
    {
        for (int i = 0; i < m_possibleBoardSizes.length; ++i)
            if (m_possibleBoardSizes[i] == size)
            {
                m_itemBoardSize[i].setSelected(true);
                return;
            }
        m_itemBoardSizeOther.setSelected(true);
    }

    public void setBeepAfterMove(boolean enable)
    {
        m_itemBeepAfterMove.setSelected(enable);
    }

    public void setCommandInProgress(boolean isInterruptSupported)
    {
        assert( !m_computer1Detached || !m_computer2Detached );
        disableAll();
        m_menuFile.setEnabled(true);
        m_itemExit.setEnabled(true);
        m_menuGame.setEnabled(true);
        if (isInterruptSupported)
            m_itemInterrupt.setEnabled(true);
        m_menuComputerColor.setEnabled(true);
        m_menuHelp.setEnabled(true);
        m_itemAbout.setEnabled(true);
        m_itemHelp.setEnabled(true);
        m_menuSettings.setEnabled(true);
        m_itemBeepAfterMove.setEnabled(true);
        //m_itemShowCursor.setEnabled(true);
        m_itemShowLastMove.setEnabled(true);
        m_menuWindow.setEnabled(true);
        m_itemGtpShell.setEnabled(true);
        //m_itemGameTree.setEnabled(true); // does not work during think (game tree cannot be shown)
        m_menu2ndGTP.setEnabled(true);
    }

    public void setComputerEnabled(boolean attached1, boolean attached2)
    {
        m_computer1Detached = !attached1;
        m_computer2Detached = !attached2;

        m_menuComputerColor.setEnabled(attached1||attached2);
        m_itemComputerPlay.setEnabled(attached1||attached2);
        m_itemBeepAfterMove.setEnabled(attached1||attached2);

        m_itemAttachProgram.setEnabled(!attached1);
        m_itemReAttachProgram.setEnabled(attached1);
        m_itemDetachProgram.setEnabled(attached1);

        m_itemAttachProgram2.setEnabled(!attached2);
        m_itemReAttachProgram2.setEnabled(attached2);
        m_itemDetachProgram2.setEnabled(attached2);

        //m_itemReAttachProgram3.setEnabled(true); //enabled&&en2
    }

    public void setNormalMode()
    {
        enableAll();
        m_itemInterrupt.setEnabled(false);
        setComputerEnabled(!m_computer1Detached, !m_computer2Detached);
        /*
        m_itemAttachProgram.setEnabled(false);
        m_itemAttachProgram2.setEnabled(false);
        if (m_computer1Detached && m_computer2Detached) {
            m_menuComputerColor.setEnabled(false);
            m_itemComputerPlay.setEnabled(false);
            m_itemBeepAfterMove.setEnabled(false);
        }
        //m_itemReAttachProgram3.setEnabled(true); //!(m_isComputerDisabled1 && m_isComputerDisabled2)
        if (m_computer1Detached) {
          m_itemAttachProgram.setEnabled(true);
          m_itemReAttachProgram.setEnabled(false);
          m_itemDetachProgram.setEnabled(false);
        }
        if (m_computer2Detached) {
          m_itemAttachProgram2.setEnabled(true);
          m_itemReAttachProgram2.setEnabled(false);
          m_itemDetachProgram2.setEnabled(false);
        }*/
    }

    public void setSetupMode()
    {
        disableAll();
        m_menuFile.setEnabled(true);
        m_menuWindow.setEnabled(true);
        m_menuHelp.setEnabled(true);
        m_itemAbout.setEnabled(true);
        m_itemExit.setEnabled(true);
        m_itemGtpShell.setEnabled(true);
        m_itemHelp.setEnabled(true);
    }

    public void setScoreMode()
    {
        disableAll();
        m_menuFile.setEnabled(true);
        m_menuWindow.setEnabled(true);
        m_menuHelp.setEnabled(true);
        m_itemAbout.setEnabled(true);
        m_itemExit.setEnabled(true);
        m_itemGtpShell.setEnabled(true);
        m_itemHelp.setEnabled(true);
    }

    /*
    public void setShowCursor(boolean enable)
    {
        m_itemShowCursor.setSelected(enable);
    }
    */

    public void setShowLastMove(boolean enable)
    {
        m_itemShowLastMove.setSelected(enable);
    }

    public void updateGameMenuItems(GameTree gameTree, Node node)
    {
        boolean hasFather = (node.getFather() != null);
        boolean hasChildren = (node.getNumberChildren() > 0);
        boolean hasNextVariation = (node.getNextVariation() != null);
        boolean hasPreviousVariation = (node.getPreviousVariation() != null);
        boolean isInMain = node.isInMainVariation();
        boolean treeHasVariations = gameTree.hasVariations();
        m_itemBeginning.setEnabled(hasFather);
        m_itemBackward.setEnabled(hasFather);
        m_itemBackward10.setEnabled(hasFather);
        m_itemForward.setEnabled(hasChildren);
        m_itemForward10.setEnabled(hasChildren);
        m_itemEnd.setEnabled(hasChildren);
        m_itemGoto.setEnabled(hasFather || hasChildren);
        m_itemNextVariation.setEnabled(hasNextVariation);
        m_itemPreviousVariation.setEnabled(hasPreviousVariation);
        m_itemBackToMainVar.setEnabled(! isInMain);
        m_itemTruncate.setEnabled(hasFather);
        m_itemMakeMainVar.setEnabled(! isInMain);
        m_itemKeepOnlyMainVar.setEnabled(isInMain && treeHasVariations);
        //m_itemKeepOnlyPosition.setEnabled(hasFather || hasChildren);
    }

    private boolean m_computer1Detached;
    private boolean m_computer2Detached;

    private static final int m_maxRecent = 20;

    private int m_numberRecent;

    private static int m_possibleBoardSizes[] = { 4,5,6,7,8,9,10,11,12,13,14,15 }; //,16,17,18,19

    private static int m_possibleLevels[] = { 0, 1, 2, 3 };

    /*
    private static int m_possibleHandicaps[] = { 0 };
    //{ 0, 2, 3, 4, 5, 6, 7, 8, 9 };
     */

    private static final int m_shortcutKeyMask =
        Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();

    private ActionListener m_listener;

    private File[] m_recent = new File[m_maxRecent];

    private File m_selectedRecent;

    private JCheckBoxMenuItem m_itemBeepAfterMove;

    //private JCheckBoxMenuItem m_itemShowCursor;

    private JCheckBoxMenuItem m_itemShowLastMove;

    private JCheckBoxMenuItem m_itemSetup;

    private JMenu m_menuComputerColor;
    private JMenu m_menuComputer2Color;

    private JMenu m_menuFile;

    private JMenu m_menuGame;

    private JMenu m_menuHelp;

    private JMenu m_menuRecent;

    private JMenu m_menuSettings;

    private JMenu m_menuSetup;

    private JMenu m_menuVariation;

    private JMenuBar m_menuBar;

    private JMenu m_menuWindow;
    private JMenu m_menu2ndGTP;

    private JMenuItem m_itemAbout;

    // ******************
    private JMenuItem m_itemAttachProgram;
    private JMenuItem m_itemReAttachProgram;
    private JMenuItem m_itemDetachProgram;

    private JMenuItem m_itemAttachProgram2;
    private JMenuItem m_itemReAttachProgram2;
    private JMenuItem m_itemDetachProgram2;

    private JMenuItem m_itemReAttachProgram3;
    // ******************

    private JMenuItem m_itemBackToMainVar;

    private JMenuItem m_itemBackward;

    private JMenuItem m_itemBackward10;

    private JMenuItem m_itemBeginning;

    private JMenuItem[] m_itemBoardSize;

    private JMenuItem[] m_itemLevel; // NEW

    private JMenuItem m_itemBoardSizeOther;

    private JMenuItem m_itemComputerBlack;

    private JMenuItem m_itemComputerBoth;

    private JMenuItem m_itemComputerNone;

    private JMenuItem m_itemComputerPlay;

    private JMenuItem m_itemComputerWhite;

    private JMenuItem m_itemComputerWhite2;
    private JMenuItem m_itemComputerBlack2;

    private JMenuItem m_itemEnd;

    private JMenuItem m_itemExit;

    private JMenuItem m_itemForward;

    private JMenuItem m_itemForward10;

    private JMenuItem m_itemGoto;

    private JMenuItem m_itemGameTree;
    private JMenuItem m_itemAnalyze;
    private JMenuItem m_itemGtpShell;

    private JMenuItem m_itemHelp;

    private JMenuItem m_itemInterrupt;

    private JMenuItem m_itemKeepOnlyMainVar;

    //private JMenuItem m_itemKeepOnlyPosition;

    private JMenuItem m_itemMakeMainVar;

    private JMenuItem m_itemNextVariation;

    private JMenuItem m_itemPreviousVariation;

    private JMenuItem m_itemEnterMoves; //NEW
    private JMenuItem m_itemEnterMoves2; //NEW

    private JMenuItem m_itemSetupBlack;

    private JMenuItem m_itemSetupWhite;

    private JMenuItem m_itemTruncate;

    private JMenuItem addMenuItem(JMenu menu, JMenuItem item, String command)
    {
        item.addActionListener(m_listener);
        item.setActionCommand(command);
        menu.add(item);
        return item;
    }

    private JMenuItem addMenuItem(JMenu menu, JMenuItem item, int mnemonic,
                                  String command)
    {
        item.setMnemonic(mnemonic);
        return addMenuItem(menu, item, command);
    }

    private JMenuItem addMenuItem(JMenu menu, String label, int mnemonic,
                                  String command)
    {
        JMenuItem item = new JMenuItem(label);
        return addMenuItem(menu, item, mnemonic, command);
    }

    private JMenuItem addMenuItem(JMenu menu, String label, int mnemonic,
                                  int accel, int modifier, String command)
    {
        JMenuItem item = new JMenuItem(label);
        KeyStroke k = KeyStroke.getKeyStroke(accel, modifier);
        item.setAccelerator(k);
        return addMenuItem(menu, item, mnemonic, command);
    }

    // NEW:
    private JMenuItem addMenuItem(JMenu menu, JMenuItem item, int mnemonic,
                                  int accel, int modifier, String command)
    {
        KeyStroke k = KeyStroke.getKeyStroke(accel, modifier);
        item.setAccelerator(k);
        return addMenuItem(menu, item, mnemonic, command);
    }

    private JMenuItem addRadioItem(JMenu menu, ButtonGroup group,
                                   String label, String command)
    {
        JMenuItem item = new JRadioButtonMenuItem(label);
        group.add(item);
        return addMenuItem(menu, item, command);
    }

    private JMenuItem addRadioItem(JMenu menu, ButtonGroup group,
                                   String label, int mnemonic, String command)
    {
        JMenuItem item = new JRadioButtonMenuItem(label);
        group.add(item);
        return addMenuItem(menu, item, mnemonic, command);
    }

    // NEW:
    private JMenuItem addRadioItem(JMenu menu, ButtonGroup group,
                                   String label, int mnemonic, int accel, int modifier, String command)
    {
        JMenuItem item = new JRadioButtonMenuItem(label);
        group.add(item);
        return addMenuItem(menu, item, mnemonic, accel, modifier, command);
    }

    private JMenu createBoardSizeMenu()
    {
        JMenu menu = createMenu("New Game / Boardsize", KeyEvent.VK_B);
        ButtonGroup group = new ButtonGroup();
        int n = m_possibleBoardSizes.length;
        m_itemBoardSize = new JMenuItem[n];
        for (int i = 0; i < n; ++i)
        {
            String s = Integer.toString(m_possibleBoardSizes[i]);
            String text=s;
            if (m_possibleBoardSizes[i]==Version.defaultBoardSize) text+=" (Default)";
            JMenuItem item = addRadioItem(menu, group, text, "board-size-" + s);
            m_itemBoardSize[i] = item;
        }
        menu.addSeparator();
        JMenuItem item = addRadioItem(menu, group, "Other",
                                      "board-size-other");
        m_itemBoardSizeOther = item;
        return menu;
    }

    private JMenu createLevelMenu()
    {
      JMenu menu = createMenu("Level", KeyEvent.VK_B);
      ButtonGroup group = new ButtonGroup();
      int n = m_possibleLevels.length;
      m_itemLevel = new JMenuItem[n];
      for (int i = 0; i < n; ++i)
      {
          String s = Integer.toString(m_possibleLevels[i]);
          String text=s;
          if (m_possibleLevels[i]==0) text+=" (Beginner)";
          if (m_possibleLevels[i]==3) text+=" (Expert)";
          JMenuItem item = addRadioItem(menu, group, text, "level-" + s);
          m_itemLevel[i] = item;
      }
      return menu;
    }

    private JMenu createComputerColorMenu()
    {
        ButtonGroup group = new ButtonGroup();
        JMenu menu = createMenu("Computer Color", KeyEvent.VK_C);
        m_itemComputerBlack = addRadioItem(menu, group, "Black", KeyEvent.VK_B,
                                           "computer-black");
        m_itemComputerWhite = addRadioItem(menu, group, "White", KeyEvent.VK_W,
                                           "computer-white");
        //m_itemComputerBoth = addRadioItem(menu, group, "Both", KeyEvent.VK_T, "computer-both");
        m_itemComputerBoth = addRadioItem(menu, group, "Both", KeyEvent.VK_T, KeyEvent.VK_F7, 0, "computer-both");
        m_itemComputerNone = addRadioItem(menu, group, "None", KeyEvent.VK_N, "computer-none");
        return menu;
    }

    private JMenu createGtp2ColorMenu()
    {
        ButtonGroup group = new ButtonGroup();
        JMenu menu = createMenu("2nd GTP - Color", KeyEvent.VK_G);
        m_itemComputerBlack2 = addRadioItem(menu, group, "Black", KeyEvent.VK_B, "2nd-black");
        m_itemComputerWhite2 = addRadioItem(menu, group, "White", KeyEvent.VK_W, "2nd-white");
        return menu;
    }

    private JMenu createFileMenu()
    {
        JMenu menu = createMenu("File", KeyEvent.VK_F);
        addMenuItem(menu, "New Game", KeyEvent.VK_N, KeyEvent.VK_N,
                    m_shortcutKeyMask, "new-game");
        menu.add(createBoardSizeMenu());
        addMenuItem(menu, "Open...", KeyEvent.VK_O, KeyEvent.VK_O,
                    m_shortcutKeyMask, "open");
        menu.add(createRecentMenu());
        addMenuItem(menu, "Save...", KeyEvent.VK_S, "save1");
        addMenuItem(menu, "Save As...", KeyEvent.VK_V, KeyEvent.VK_S, m_shortcutKeyMask, "save");
        addMenuItem(menu, "Save As (Auto filename)...", KeyEvent.VK_U, "save-auto");
        //addMenuItem(menu, "Save Position...", KeyEvent.VK_T, "save-position");
        //menu.addSeparator();
        //addMenuItem(menu, "Print...", KeyEvent.VK_P, KeyEvent.VK_P, m_shortcutKeyMask, "print");
        menu.addSeparator();

        menu.add(createLevelMenu());
        menu.addSeparator();

        //m_itemAttachProgram = addMenuItem(menu, "Attach Program...", KeyEvent.VK_A, "attach-program");
        //m_itemReAttachProgram = addMenuItem(menu, "ReAttach Program...", KeyEvent.VK_C, "reattach-program");
        //m_itemDetachProgram = addMenuItem(menu, "Detach Program", KeyEvent.VK_D, "detach-program");
        m_itemAttachProgram = addMenuItem(menu, "Attach Program...", KeyEvent.VK_A, KeyEvent.VK_F2, 0, "attach-program");
        m_itemReAttachProgram = addMenuItem(menu, "ReAttach Program...", KeyEvent.VK_C, KeyEvent.VK_F3, 0, "reattach-program");
        m_itemDetachProgram = addMenuItem(menu, "Detach Program", KeyEvent.VK_D, KeyEvent.VK_F4, 0, "detach-program");
        menu.addSeparator();
        m_itemExit = addMenuItem(menu, "Quit", KeyEvent.VK_Q, KeyEvent.VK_Q,
                                 m_shortcutKeyMask, "exit");
        return menu;
    }

    private JMenu createGameMenu()
    {
        JMenu menu = createMenu("Game", KeyEvent.VK_G);
        addMenuItem(menu, "Game Information...", KeyEvent.VK_I, KeyEvent.VK_I,
                    m_shortcutKeyMask, "game-info");
        //menu.add(createHandicapMenu());
        m_menuComputerColor = createComputerColorMenu();
        menu.add(m_menuComputerColor);
        menu.addSeparator();
        //addMenuItem(menu, "Pass", KeyEvent.VK_P, KeyEvent.VK_F2, 0, "pass");
        //addMenuItem(menu, "Swap", KeyEvent.VK_W, KeyEvent.VK_F7, 0, "swap");
        addMenuItem(menu, "Swap", KeyEvent.VK_W, KeyEvent.VK_W, m_shortcutKeyMask, "swap");
        m_itemComputerPlay = addMenuItem(menu, "Computer Play", KeyEvent.VK_L,
                                         KeyEvent.VK_F5, 0, "play");
        m_itemInterrupt =
            addMenuItem(menu, "Interrupt", KeyEvent.VK_T, KeyEvent.VK_ESCAPE,
                        0, "interrupt");
        menu.addSeparator();
        m_itemBeginning =
            addMenuItem(menu, "Beginning", KeyEvent.VK_N, KeyEvent.VK_HOME,
                        m_shortcutKeyMask, "beginning");
        m_itemBackward10 =
            addMenuItem(menu, "Backward 10", KeyEvent.VK_D, KeyEvent.VK_LEFT,
                        m_shortcutKeyMask | ActionEvent.SHIFT_MASK,
                        "backward-10");
        m_itemBackward =
            addMenuItem(menu, "Backward", KeyEvent.VK_B, KeyEvent.VK_LEFT,
                        m_shortcutKeyMask, "backward");
        m_itemForward =
            addMenuItem(menu, "Forward", KeyEvent.VK_F, KeyEvent.VK_RIGHT,
                        m_shortcutKeyMask, "forward");
        m_itemForward10 =
            addMenuItem(menu, "Forward 10", KeyEvent.VK_O, KeyEvent.VK_RIGHT,
                        m_shortcutKeyMask | ActionEvent.SHIFT_MASK,
                        "forward-10");
        m_itemEnd =
            addMenuItem(menu, "End", KeyEvent.VK_E, KeyEvent.VK_END,
                        m_shortcutKeyMask, "end");
        m_itemGoto =
            addMenuItem(menu, "Go To...", KeyEvent.VK_G, KeyEvent.VK_G,
                        m_shortcutKeyMask, "goto");
        return menu;
    }

    /*
    private JMenu createHandicapMenu()
    {
        JMenu menu = createMenu("Handicap [unused]", KeyEvent.VK_H);
        ButtonGroup group = new ButtonGroup();
        for (int i = 0; i < m_possibleHandicaps.length; ++i)
        {
            String s = Integer.toString(m_possibleHandicaps[i]);
            JMenuItem item = addRadioItem(menu, group, s, "handicap-" + s);
            if (m_possibleHandicaps[i] == 0)
                item.setSelected(true);
        }
        return menu;
    }
    */

    private JMenu create2ndGTPMenu()
    {
        JMenu menu = createMenu("2nd GTP", KeyEvent.VK_N);
        m_menuComputer2Color=createGtp2ColorMenu();
        m_itemAttachProgram2  = addMenuItem(menu, "Attach 2nd GTP...", KeyEvent.VK_A, "attach-program2");
        m_itemReAttachProgram2= addMenuItem(menu, "ReAttach 2nd GTP...", KeyEvent.VK_C, "reattach-program2");
        m_itemDetachProgram2  = addMenuItem(menu, "Detach 2nd GTP", KeyEvent.VK_D, "detach-program2");
        menu.addSeparator();
        m_itemReAttachProgram3= addMenuItem(menu, "ReAttach both...", KeyEvent.VK_C, "reattach-program3");
        return menu;
    }

    private JMenu createHelpMenu()
    {
        JMenu menu = createMenu("Help", KeyEvent.VK_H);
        JMenuItem itemHelp =
          //addMenuItem(menu, "HexGui Documentation", KeyEvent.VK_C, KeyEvent.VK_F1, 0, "help");
            addMenuItem(menu, "HexGui Documentation", KeyEvent.VK_C, "help");
        JMenuItem itemAbout = addMenuItem(menu, "About", KeyEvent.VK_A, "about");
        m_itemHelp = itemHelp;
        m_itemAbout = itemAbout;
        return menu;
    }

    private JMenu createMenu(String name, int mnemonic)
    {
        JMenu menu = new JMenu(name);
        menu.setMnemonic(mnemonic);
        return menu;
    }

    private JMenu createRecentMenu()
    {
        m_menuRecent = createMenu("Open Recent", KeyEvent.VK_R);
        loadRecent();
        updateRecentMenu();
        return m_menuRecent;
    }

    private JMenu createSettingsMenu()
    {
        JMenu menu = createMenu("Settings", KeyEvent.VK_S);
        /*
        m_itemShowCursor = new JCheckBoxMenuItem("Show Cursor");
        m_itemShowCursor.setSelected(true);
        addMenuItem(menu, m_itemShowCursor, KeyEvent.VK_S, "show-cursor");
        */
        m_itemShowLastMove = new JCheckBoxMenuItem("Show Last Move");
        m_itemShowLastMove.setSelected(true);
        addMenuItem(menu, m_itemShowLastMove, KeyEvent.VK_L, "show-last-move");

        m_itemBeepAfterMove = new JCheckBoxMenuItem("Beep After Move"); //(after computer moves only)
        addMenuItem(menu, m_itemBeepAfterMove, KeyEvent.VK_B,
                    "beep-after-move");
        return menu;
    }

    private JMenu createSetupMenu()
    {
        JMenu menu = createMenu("Setup", KeyEvent.VK_E);
        m_itemSetup = new JCheckBoxMenuItem("Setup Mode");
        addMenuItem(menu, m_itemSetup, KeyEvent.VK_S, "setup");
        menu.addSeparator();
        ButtonGroup group = new ButtonGroup();
        m_itemSetupBlack =
            addRadioItem(menu, group, "Black", KeyEvent.VK_B, "setup-black");
        m_itemSetupBlack.setSelected(true);
        m_itemSetupWhite =
            addRadioItem(menu, group, "White", KeyEvent.VK_W, "setup-white");
        return menu;
    }

    private JMenu createVariationMenu()
    {
        JMenu menu = createMenu("Variation", KeyEvent.VK_V);
        //Enter Moves (as  text) ...
        m_itemEnterMoves = addMenuItem(menu, "Enter Moves (sequence) ...", KeyEvent.VK_E, "enter-moves"); //NEW
        m_itemEnterMoves2 = addMenuItem(menu, "Enter Moves (children) ...", KeyEvent.VK_H, "enter-moves2"); //NEW
        m_itemNextVariation =
            addMenuItem(menu, "Next Variation", KeyEvent.VK_N,
                        KeyEvent.VK_DOWN, m_shortcutKeyMask,
                        "next-variation");
        m_itemPreviousVariation =
            addMenuItem(menu, "Previous Variation", KeyEvent.VK_P,
                        KeyEvent.VK_UP, m_shortcutKeyMask,
                        "previous-variation");
        m_itemBackToMainVar =
            addMenuItem(menu, "Back to Main Variation", KeyEvent.VK_B,
                        KeyEvent.VK_M, m_shortcutKeyMask,
                        "back-to-main-variation");
        m_itemMakeMainVar = addMenuItem(menu, "Make Main Variation",
                                        KeyEvent.VK_M, "make-main-variation");
        m_itemKeepOnlyMainVar = addMenuItem(menu,
                                            "Delete Side Variations",
                                            KeyEvent.VK_D,
                                            "keep-only-main-variation");
        /*m_itemKeepOnlyPosition = addMenuItem(menu,
                                             "Keep Only Position",
                                             KeyEvent.VK_K,
                                             "keep-only-position");*/
        //m_itemTruncate = addMenuItem(menu, "Truncate", KeyEvent.VK_T,"truncate");
        m_itemTruncate =
            addMenuItem(menu, "Truncate", KeyEvent.VK_T,
                        KeyEvent.VK_T, m_shortcutKeyMask,
                        "truncate");
        return menu;
    }

    private JMenu createWindowMenu()
    {
        JMenu menu = createMenu("Window", KeyEvent.VK_W);
        m_itemGameTree = addMenuItem(menu, "Game Tree", KeyEvent.VK_T, KeyEvent.VK_F9, 0, "show-gametree");
        m_itemAnalyze = addMenuItem(menu, "Analyze", KeyEvent.VK_A, KeyEvent.VK_F11, 0, "analyze");
        m_itemGtpShell = addMenuItem(menu, "GTP Shell(s)", KeyEvent.VK_G, KeyEvent.VK_F12, 0, "gtp-shell");
        menu.addSeparator();
        addMenuItem(menu, "Bring All to Front", KeyEvent.VK_F, "all-to-front");
        return menu;
    }

    private void disableAll()
    {
        for (int i = 0; i < m_menuBar.getMenuCount(); ++i)
        {
            JMenu menu = m_menuBar.getMenu(i);
            if (menu != null)
                disableMenu(menu);
        }
    }

    private void disableMenu(JMenu menu)
    {
        menu.setEnabled(false);
        for (int i = 0; i < menu.getItemCount(); ++i)
            if (menu.getItem(i) != null)
                menu.getItem(i).setEnabled(false);
    }

    private void enableAll()
    {
        for (int i = 0; i < m_menuBar.getMenuCount(); ++i)
        {
            JMenu menu = m_menuBar.getMenu(i);
            if (menu != null)
            {
                menu.setEnabled(true);
                for (int j = 0; j < menu.getItemCount(); ++j)
                    if (menu.getItem(j) != null)
                        menu.getItem(j).setEnabled(true);
            }
        }
        m_menuRecent.setEnabled(m_numberRecent > 0);
    }

    private File getRecentFile()
    {
        String home = System.getProperty("user.home");
        return new File(new File(home, version.Version.PropertyDir()), "recent-files");
    }

    private void loadRecent()
    {
        m_numberRecent = 0;
        File file = getRecentFile();
        BufferedReader reader;
        try
        {
            reader = new BufferedReader(new FileReader(file));
        }
        catch (FileNotFoundException e)
        {
            return;
        }
        String line;
        try
        {
            while((line = reader.readLine()) != null)
            {
                if (m_numberRecent >= m_maxRecent - 1)
                    break;
                File recent = new File(line);
                if (! recent.exists())
                    continue;
                m_recent[m_numberRecent] = recent;
                ++m_numberRecent;
            }
        }
        catch (IOException e)
        {
        }
        try
        {
            reader.close();
        }
        catch (IOException e)
        {
        }
    }

    private void updateRecentMenu()
    {
        m_menuRecent.removeAll();
        m_numberRecent = 0;
        for (int i = 0; i < m_maxRecent; ++i)
        {
            File file = m_recent[i];
            if (file == null)
                break;
            ++m_numberRecent;
            JMenuItem item = new JMenuItem(file.getName());
            item.addActionListener(this);
            item.setActionCommand("open-recent-" + i);
            m_menuRecent.add(item);
        }
        m_menuRecent.setEnabled(m_numberRecent > 0);
    }
}

//----------------------------------------------------------------------------
