//----------------------------------------------------------------------------
// $Id: Writer.java,v 1.28 2004/05/10 22:49:07 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/sgf/Writer.java,v $
//----------------------------------------------------------------------------

package sgf;

//----------------------------------------------------------------------------

import java.io.*;
import java.util.*;
import game.*;
import go.*;

//----------------------------------------------------------------------------

public class Writer
{
    public static class Error extends Exception
    {
        public Error(String s)
        {
            super(s);
        }
    }

    /** Save game tree. */
    public Writer(OutputStream out, GameTree gameTree, File file,
                  String application, String version)
    {
        m_out = new PrintStream(out);
        m_out_println("(");
        GameInformation gameInformation = gameTree.getGameInformation();
        m_size = gameInformation.m_boardSize;
        String result = gameInformation.m_result;
        String playerBlack = gameInformation.m_playerBlack;
        String playerWhite = gameInformation.m_playerWhite;
        String rankBlack = gameInformation.m_blackRank;
        String rankWhite = gameInformation.m_whiteRank;
        String date = gameInformation.m_date;
        String rules = gameInformation.m_rules;
        int handicap = gameInformation.m_handicap;
        float komi = gameInformation.m_komi;
        printHeader(file, application, version, handicap, date, playerBlack,
                    playerWhite, rankBlack, rankWhite, result, komi, rules);
        printNodes(gameTree.getRoot());
        m_out.println(")");
        m_out.close();
    }

    /** Save board position. */
    public Writer(OutputStream out, GameBoard board, File file,
                  String application, String version)
    {
        m_size = board.getSize();
        m_out = new PrintStream(out);
        m_out_println("(");
        printHeader(file, application, version);
        printPosition(board);
        m_out.println(")");
        m_out.close();
    }

    private int m_size;

    private PrintStream m_out;

    private static String getName(File file)
    {
        String result = file.getName();
        int len = result.length();
        if (len >= 4
            && result.substring(len - 4).toLowerCase().equals(".sgf"))
            result = result.substring(0, len - 4);
        return result;
    }

    private String esc(String s)
    {
        String s2="";
        for (int i = 0; i < s.length(); ++i)
        {
            char c = s.charAt(i);
            if ("]:\\".indexOf(c) >= 0) { //escape!
              s2+='\\';
              s2+=c;
            }
            //else if (c != '\n' && Character.isWhitespace(c))
            //  s2+=' ';
            else
              s2+=c;
        }
        return s2;
    }

    private void printEscaped(String text)
    {
        m_out.print('[');
        m_out.print(esc(text));
        m_out.print(']');
    }

    void m_out_println(String s) {
        m_out.print(s);
        //m_out.println(s);
    }
    void m_out_println() {
        //m_out.println();
    }

    private void printHeader(File file, String application, String version)
    {
        String appName = application;
        //if (version != null && ! version.equals(""))
        //    appName = appName + ":" + version;
        m_out_println(";");
        m_out_println("FF[4]" +   // FF FileFormat 4
                      "GM[Hex]" + // GM Gametype "Hex" / 11
                      //"AP[HexGui:0.6.4]" +  // Application
                      ""
                      //"GN[" + getName(file) + "]\n" + // game name
        );
        m_out_println("SZ[" + m_size + "]");
    }

    private void printHeader(File file, String application, String version,
                             int handicap, String date, String playerBlack,
                             String playerWhite, String rankBlack,
                             String rankWhite, String result, float komi,
                             String rules)
    {
        printHeader(file, application, version);
        if ((playerBlack != null)&&(!playerBlack.equals("")))
            m_out_println("PB[" + esc(playerBlack) + "]");
        if ((rankBlack != null)&&(!rankBlack.equals("")))
            m_out_println("BR[" + esc(rankBlack) + "]");
        if ((playerWhite != null)&&(!playerWhite.equals("")))
            m_out_println("PW[" + esc(playerWhite) + "]");
        if ((rankWhite != null)&&(!rankWhite.equals("")))
            m_out_println("WR[" + esc(rankWhite) + "]");
        if (result != null && ! result.equals(""))
            m_out_println("RE[" + esc(result) + "]");
        if (rules != null && ! rules.equals(""))
            m_out_println("RU[" + esc(rules) + "]");
        //if (date != null)
        //  m_out_println("DT[" + date + "]");
    }

    private void printNodes(Node node)
    {
        Move move = node.getMove();
        if (move != null)
        {
            if (move.getColor() == Color.BLACK) {
                m_out_println(";");
                m_out.print("B");
            } else {
                m_out_println(";");
                m_out.print("W");
            }
            printMove(move);
            m_out_println();
        }
        if (node.getNumberAddBlack() > 0)
        {
            m_out.print("AB");
            for (int i = 0; i < node.getNumberAddBlack(); ++i)
                printPoint(node.getAddBlack(i));
            m_out_println();
        }
        if (node.getNumberAddWhite() > 0)
        {
            m_out.print("AW");
            for (int i = 0; i < node.getNumberAddWhite(); ++i)
                printPoint(node.getAddWhite(i));
            m_out_println();
        }
        String comment = node.getComment();
        if (comment != null && ! comment.trim().equals(""))
        {
            m_out.print("C");
            printEscaped(comment);
            m_out_println();
        }
        if (! Float.isNaN(node.getTimeLeftBlack()))
        {
            m_out_println("BL[" + node.getTimeLeftBlack() + "]");
        }
        if (node.getMovesLeftBlack() >= 0)
        {
            m_out_println("OB[" + node.getMovesLeftBlack() + "]");
        }
        if (! Float.isNaN(node.getTimeLeftWhite()))
        {
            m_out_println("WL[" + node.getTimeLeftWhite() + "]");
        }
        if (node.getMovesLeftWhite() >= 0)
        {
            m_out_println("OW[" + node.getMovesLeftWhite() + "]");
        }
        if (node.getPlayer() != Color.EMPTY)
            printToPlay(node.getPlayer());
        Map sgfProperties = node.getSgfProperties();
        if (sgfProperties != null)
        {
            Iterator it = sgfProperties.entrySet().iterator();
            while (it.hasNext())
            {
                Map.Entry entry = (Map.Entry)it.next();
                String label = (String)entry.getKey();
                String value = (String)entry.getValue();
                m_out.print(label);
                printEscaped(value);
                m_out_println();
            }
        }
        int numberChildren = node.getNumberChildren();
        if (numberChildren == 0)
            return;
        if (numberChildren == 1)
        {
            printNodes(node.getChild());
            return;
        }
        for (int i = 0; i < numberChildren; ++i)
        {
            m_out_println("(");
            printNodes(node.getChild(i));
            m_out_println(")");
        }
    }

    private void printPoint(Point p)
    {
        if (p == null) {
            m_out.print("[]"); // special ; should not happen ?!
            return;
        }
        m_out.print("[" + (char)('a' + p.getX()) + (p.getY()+1) + "]");
    }

    // NEW:
    // * for writing to sgf
    private void printMove(Move m)
    {
        m_out.print("[" + m.toString(true) + "]");
    }

    private void printPointList(Vector v)
    {
        for (int i = 0; i < v.size(); ++i)
            printPoint((Point)v.get(i));
    }

    private void printPosition(GameBoard board)
    {
        int numberPoints = board.getNumberPoints();
        Vector black = new Vector(numberPoints);
        Vector white = new Vector(numberPoints);
        for (int i = 0; i < numberPoints; ++i)
        {
            Point p = board.getPoint(i);
            Color c = board.getColor(p);
            if (c == Color.BLACK)
                black.add(p);
            else if (c == Color.WHITE)
                white.add(p);
        }
        printSetup(black, white);
        printToPlay(board.getToMove());
    }

    private void printSetup(Vector black, Vector white)
    {
        if (black.size() > 0 || white.size() > 0)
        {
            if (black.size() > 0)
            {
                m_out.print("AB");
                printPointList(black);
                m_out_println();
            }
            if (white.size() > 0)
            {
                m_out.print("AW");
                printPointList(white);
                m_out_println();
            }
        }
    }

    // only for position:
    private void printToPlay(Color color)
    {
        if (color == Color.BLACK)
            m_out_println("PL[B]");
        else
            m_out_println("PL[W]");
    }
}

//----------------------------------------------------------------------------
