//----------------------------------------------------------------------------
// $Id: SgfToTex.java,v 1.17 2004/05/10 22:49:07 enz Exp $
// $Source: /cvsroot/gogui/gogui/src/sgftotex/SgfToTex.java,v $
//----------------------------------------------------------------------------

package sgftotex;

import java.io.*;
import java.util.*;
import utils.*;
import version.*;

//----------------------------------------------------------------------------

class SgfToTex
{
    public static void main(String[] args)
    {
        try
        {
            String options[] = {
                "config:",
                "force",
                "help",
                "pass",
                "title:",
                "version"
            };
            Options opt = new Options(args, options);
            opt.handleConfigOption();
            if (opt.isSet("help"))
            {
                printUsage(System.out);
                System.exit(0);
            }
            if (opt.isSet("version"))
            {
                System.out.println("SgfToTex " + Version.getVersion());
                System.exit(0);
            }
            boolean usePass = opt.isSet("pass");
            boolean force = opt.isSet("force");
            String title = opt.getString("title", "");
            Vector arguments = opt.getArguments();
            InputStream in;
            OutputStream out;
            if (arguments.size() > 2)
            {
                printUsage(System.err);
                System.exit(-1);
            }
            if (arguments.size() == 0)
            {
                in = System.in;
                out = System.out;
            }
            else
            {
                String inFileName = (String)arguments.get(0);
                File inFile = new File(inFileName);
                in = new FileInputStream(inFile);
                String outFileName;
                if (arguments.size() == 1)
                    outFileName =
                        FileUtils.replaceExtension(inFile, "sgf", "tex");
                else
                    outFileName = (String)arguments.get(1);
                File outFile = new File(outFileName);
                if (outFile.exists() && ! force)
                    throw new Exception("File " + outFile
                                        + " already exists");
                out = new FileOutputStream(outFile);
                if (title.equals(""))
                    title =
                        FileUtils.removeExtension(new File(outFile.getName()),
                                                  "tex");
            }
            convert(in, out, title, usePass);
        }
        catch (AssertionError e)
        {
            e.printStackTrace();
            System.exit(-1);
        }
        catch (RuntimeException e)
        {
            System.err.println(StringUtils.formatException(e));
            e.printStackTrace();
            System.exit(-1);
        }
        catch (Throwable t)
        {
            System.err.println(StringUtils.formatException(t));
            System.exit(-1);
        }
    }

    private static void convert(InputStream in, OutputStream out,
                                String title, boolean usePass)
        throws sgf.Reader.Error
    {
        sgf.Reader reader = new sgf.Reader(new InputStreamReader(in), null);
        //new latex.Writer(title, out, reader.getGameTree(), usePass);
    }

    private static void printUsage(PrintStream out)
    {
        out.print("Usage: java -jar sgftotex.jar [file.sgf [file.tex]]\n" +
                  "\n" +
                  "-config  config file\n" +
                  "-force   overwrite existing files\n" +
                  "-help    display this help and exit\n" +
                  "-pass    use \\pass command\n" +
                  "-title   use title\n" +
                  "-version print version and exit\n");
    }
}

//----------------------------------------------------------------------------
