#include "hexmatch.h"
#include "sixplayer.h"
#include "misc.h"

#ifdef HAVE_BOOST_THREADS
#include <boost/thread/thread.hpp>
using namespace boost;
using namespace boost::posix_time;
#endif

#include <iostream>
#include <fstream>
#include <sstream>

typedef std::ostringstream OSS;

using namespace std;

//typedef unsigned long      uint32;
typedef unsigned long long uint64;
typedef unsigned long long ull;

#ifndef __VSIZE__
# error vsize needs to be defined!!
#elif __VSIZE__ == 4
 const int boardsize_min=4;
 const int boardsize_max=11;
#elif __VSIZE__ == 8
 const int boardsize_min=12; //smaller sizes are blocked (because too slow)
 const int boardsize_max=15;
#else
# error vsize wrong!!
#endif

#ifdef HAVE_WINDOWS_H

#include <windows.h>

uint64 currentTimeMillis() {
  // WINDOWS:
  SYSTEMTIME st;
  FILETIME ft;
  //ULARGE_INTEGER res;

  GetSystemTime(&st); // UCT

  BOOL b=SystemTimeToFileTime(&st, &ft);
  if (b==0) return 0;

  uint64 res=*(uint64*)&ft;

  res/=10000; // 100-nanosecond intervals TO ms-intervals // since January 1, 1601 (UTC)
  return res;
}

//e.g. setThreadPrio(THREAD_PRIORITY_LOWEST);
bool setThreadPrio(int newPrio) {
  HANDLE hThread=GetCurrentThread(); // pseudo handle of current thread
  BOOL newPrioOk=SetThreadPriority(hThread, newPrio);
  return newPrioOk;
}

#else

bool setThreadPrio(int newPrio) {
  return pthread_setschedprio(pthread_self(), newPrio) == 0;
}

#endif


// includes for implementation
#include <cstdlib>
#include <ctime>
#include <iostream>
#include <fstream>
#include <string>

using namespace std;

bool isDigit(char c) {
	return c>='0' && c<='9';
}

//int min_length=1;
bool isPositiveNumber(const string& line) {
    if (line.length()==0) return false;
	for (int i=0; i<line.length(); i++)
		if (!isDigit(line[i]))
			return false;
	return true;
}

// check global variable parseOk after calling!
bool parseOk;
int parseInt(const char* s) {
  char *tail;
  int l=strtol(s,&tail,10);
  parseOk=(*tail=='\0');
  return l;
}

// check global variable parseOk after calling!
int parseInt(const string& line) {
  return parseInt(line.c_str());
}

string getParam(const string& line, int nr) {
  int cnt=0;
  size_t begin=0;
  while (cnt<nr) {
    begin=line.find(" ",begin);
    if (begin==-1) return string("");
    begin++;
	while ((begin<line.length())&&(line[begin]==' ')) begin++; // skip multiple spaces
    cnt++;
  }
  size_t end=line.find(" ",begin);
  if (end==-1) end=line.length();
  return line.substr(begin,end-begin);
}

// only positive
int getParamInt(const string& line, int nr) {
  string s1=getParam(line,nr);
  if (!isPositiveNumber(s1)) return -9999;
  int i1=parseInt(s1);
  if (parseOk)
	  return i1;
  else
	  return -9999;
}

int level=0; //0,1,2,3
HexMatch* m_match;

int think_state=0; //0=not thinking, 1=thinking, 2=thinking thread returned (finished or aborted)

#ifdef HAVE_WINDOWS_H
ull time1; //begin of thinking
#else
system_time time1;
#endif

/**
 *  think + write answer to stdout
 */
void think_thread() {
#ifdef HAVE_WINDOWS_H
  bool low_prio_ok=setThreadPrio(THREAD_PRIORITY_LOWEST); //lowest prio
#else
  bool low_prio_ok=setThreadPrio(sched_get_priority_min(sched_getscheduler(0))); //lowest prio
#endif

  bool res=m_match->doSome(); //*MAIN THINKING FUNCTION*
#ifdef HAVE_WINDOWS_H
  ull time2=currentTimeMillis();
  DBG << "six (level="<<level<<"; "<< (time2-time1)<<" ms)\n"; // "six (level=0; 2203 ms)"
#else
  system_time time2 = get_system_time();
  time_duration td = time2-time1;
  DBG << "six (level="<<level<<"; "<< td.total_milliseconds() <<" ms)\n"; // "six (level=0; 2203 ms)"
#endif
  DBG << "res="<<res<<endl;
  DBG_flush();
  if (res) {
    //cerr<<"doSome OK!!"<<endl;
    cout << "= ";
    m_match->game().printMove(cout, (*(m_match->game().end()-1)).move(), false);
    //m_match->game().printMove(cerr, (*(m_match->game().end()-1)).move(), false);
    //ans_done=true;
    cout << "\n"<<endl;
    //cerr<<"doSome OK!!"<<endl;
  } else {
    cout << "? interrupted: no move found yet!\n"<<endl; //<<endl //interrupted: //no move found yet!
  }

  think_state=2; //finished
}

// *****************************************************************************************
// *****************************************************************************************
//#include "main_common.h"

int main(int argc, char **argv) {
#ifdef HAVE_BOOST_THREADS
  thread* t1=NULL;
#endif
  int EXIT_ON_ERROR=0;

  // ******************************************************************************
  //# swap disallow (?)
  bool swap_allowed=true;
  bool resign_allowed=true;
  int board_size=11;

  for (int i=1; i<argc; i++) {
    if (0==strcmp(argv[i],"--level=0")) level=0;
    if (0==strcmp(argv[i],"--level=1")) level=1;
    if (0==strcmp(argv[i],"--level=2")) level=2;
    if (0==strcmp(argv[i],"--level=3")) level=3;
    if (0==strcmp(argv[i],"--allow-swap=0")) swap_allowed=0;
    if (0==strcmp(argv[i],"--allow-swap=1")) swap_allowed=1;
    if (0==strcmp(argv[i],"--allow-resign=0")) resign_allowed=0;
    if (0==strcmp(argv[i],"--allow-resign=1")) resign_allowed=1;
  }

  cerr << "level="<<level<<" (0..3)"<<endl;
  cerr << "swap_allowed="<<swap_allowed<<endl;
  cerr << "resign_allowed="<<resign_allowed<<endl;
  cerr << "boardsize: "<<boardsize_min<<".."<<boardsize_max<<endl;

  // ******************************
  Carrier::init();
  Poi<HexPlayer> Vert0(new SixPlayer((SixPlayer::LevelT)level, resign_allowed));
  Poi<HexPlayer> Horiz0(new SixPlayer((SixPlayer::LevelT)level, resign_allowed));

  HexBoard* m_board=new HexBoard(board_size,board_size);
  m_match=new HexMatch(
      HexGame(*m_board, HEX_MARK_VERT, swap_allowed),
      Vert0,
      Horiz0);

  // **************************************************************************
  // 'list_commands' ==> 'clear_board','play','genmove','quit'
  // for possible commands see code, eg:
  // "clear_board 11"
  // "play [b|w] [a17|swap|resign]" //nicht: pass (?)
  // "genmove [b|w]"

  string line;
  string ans;
  while (1) { //std::getline(cin,line)
    bool command_ok=1;
    bool ans_done=0;

    line="";

    // Read one line:
    int ch;
    while (1) {
      ch=cin.get();
      if (ch==std::char_traits<char>::eof()) return 0;
           if ((char)ch=='\r') { } // do nothing
      else if ((char)ch=='\n') { break; }
      else { line.append(1, (char)ch); } // append one character!!
    }

    if ((line.find("# interrupt")==0) || (line.find("#interrupt")==0)) {
      ans_done=1; //nothing to answer!!
#ifdef HAVE_BOOST_THREADS
      if (think_state>=1) {
        DBG<<"GOT: interrupt!!"<<endl; DBG_flush();
        m_match->cancelThinking();
        t1->join(); // wait for the thread to finish (its ok to call when thread already finished)
        delete t1; t1=NULL; think_state=0;
      } else {
        DBG<<"IGNORE: interrupt!!"<<endl; DBG_flush();
      }
#else
      DBG<<"IGNORE: interrupt (not supported!)"<<endl; DBG_flush();
#endif
    }
    else { //other command
#ifdef HAVE_BOOST_THREADS
      if (think_state>=1) { //wait without interrupting!
        t1->join(); // wait for the thread to finish (its ok to call when thread already finished)
        delete t1; t1=NULL; think_state=0;
      }
#endif

      if (line.find("name")==0) {
        cout << "= Six GTP Engine, (c) by Gabor Melis\n"<<endl; ans_done=1;
      }
      else if (line.find("protocol_version")==0) {
        cout << "= 2\n"<<endl; ans_done=1;
      }
      else if (line.find("version")==0) {
        cout << "= 0.5.3\n"<<endl; ans_done=1;
      }
#ifdef HAVE_BOOST_THREADS
      else if (line.find("gogui_interrupt")==0) {
        // ok
      }
#endif
      else if (line.find("list_commands")==0) {
        cout << "= name\nversion\nlist_commands\nplay\ngenmove\nboardsize\nclear_board\nquit\n";
#ifdef HAVE_BOOST_THREADS
        cout << "gogui_interrupt\n";
#endif
        cout<<endl; ans_done=1;
      }
      else if (line.find("clear_board")==0) { // no parameter!!
        delete m_match;
        m_match=new HexMatch(
          HexGame(*m_board, HEX_MARK_VERT, swap_allowed),
          Vert0,
          Horiz0);
      }
      else if (line.find("boardsize")==0) { // boardsize 11 for example
        int x=getParamInt(line, 1);
        if (x>=boardsize_min && x<=boardsize_max) {
          board_size=x;
          delete m_match;
          delete m_board;
          m_board=new HexBoard(board_size,board_size);
          m_match=new HexMatch(
            HexGame(*m_board, HEX_MARK_VERT, swap_allowed),
            Vert0,
            Horiz0);
        }
        else {
          command_ok=0;
          ans_done=1;
          cout << "? invalid boardsize: "<<x<<" ("<<boardsize_min<<".."<<boardsize_max<<" allowed)\n"<<endl;
        }
      }
      else if (line.find("undo")==0) {
        command_ok=m_match->canBack();
        if (command_ok) m_match->back(); else {
          ans_done=1;
          cout << "? undo impossible\n"<<endl;
        }
      }
      else if (line.find("play")==0) { // play [b|w] [a17|swap|resign] //a17 X:a...z Y:1..24
        command_ok=false;
        HexMark col=HEX_MARK_EMPTY;
        if (line.find("play black ")==0) col=HEX_MARK_VERT;
        if (line.find("play white ")==0) col=HEX_MARK_HORI;

        //cerr<<"play; col="<<col<<endl;

        // game finished?
        //cerr<<"m_match->status()="<<m_match->status()<<endl;
        if (m_match->status() == HexMatch::MATCH_FINISHED) col=HEX_MARK_EMPTY; // error: game finished

        // wrong player to move?
        //cerr<<"m_match->game().next()="<<m_match->game().next()<<endl;
        if (m_match->game().next() != col) col=HEX_MARK_EMPTY; // error: next player wrong

        //cerr<<"play; col="<<col<<endl;
        if (col != HEX_MARK_EMPTY) {
          size_t len1=string("play black ").length();
          string param2=line.substr(len1);
          //cerr<<"play; col="<<col<<endl;

          HexMove mm;
          bool move_ok=false;

               if (param2=="swap")   { mm=HexMove::createSwap(col);   move_ok=true; }
          else if (param2=="resign") { mm=HexMove::createResign(col); move_ok=true; }
          else {
            //cerr<<"play ..."<<endl;
            if (param2.length()==2 || param2.length()==3) {
              //cerr<<"param2"<<param2<<endl;
              int x=toupper(param2[0])-'A';
              int y=param2[1]-'0';
              if ((y>=0 && y<=9)) {
                if (param2.length()==3) {
                  y*=10;
                  y+=param2[2]-'0';
                }
                y--;
                //cerr<<"x"<<x;
                //cerr<<"y"<<y;
                //cerr<<endl;
                mm=HexMove(col, m_board->coords2Field(x, y)); // coords2Field has no bound-check!!
                move_ok=true;
              }
            }
          } // end-if (parse move)

          // have parsed a move
          if (move_ok) {
            if (m_match->game().isValidMove(mm)) {
              move_ok=m_match->doSome_UserMove(mm);
            }
            else move_ok=false;
          }
          command_ok=move_ok;
        } //end-if (valid color)
      } // end-if (command="play")
      else if (line.find("genmove")==0) { // genmove [b|w] //a17 X:a...z Y:1..24
        command_ok=false;
        HexMark col=HEX_MARK_EMPTY;
        if (line.find("genmove black")==0) col=HEX_MARK_VERT;
        if (line.find("genmove white")==0) col=HEX_MARK_HORI;

        // game finished?
        if (m_match->status() == HexMatch::MATCH_FINISHED) col=HEX_MARK_EMPTY; // error: game finished
        // wrong player to move?
        if (m_match->game().next() != col) col=HEX_MARK_EMPTY; // error: next player wrong

        if (col != HEX_MARK_EMPTY) {
          if (m_match->status() != HexMatch::MATCH_FINISHED) {
            DBG << "level="<<level<<endl; DBG_flush();
            //cerr<<"doSome"<<endl;
#ifdef HAVE_WINDOWS_H
            time1=currentTimeMillis();
#else
            time1=get_system_time();
#endif
#ifdef HAVE_BOOST_THREADS
            think_state=1; //"thinking"
            t1 = new thread(&think_thread); //think in new thread!
#else
            think_thread(); //simply call the function!
            think_state=0; //"not thinking"
#endif
            ans_done=true;
          }
        }
      }
      else if (line.find("quit")==0) {
        return 0;
      }
      else
        command_ok=0;
    } //<end> "other command" (than #interrupt)

    if (!ans_done) {
      if (command_ok) cout << "= \n"<<endl; // 0K
                 else cout << "? error\n"<<endl; // ERROR
    }

    if (EXIT_ON_ERROR && !command_ok) { //# only wrong clearboard/play/genmove ... commands!!
      exit(1);
    }
  } // end while (read line)

  return 0;
}
